{ This library is free software; you can redistribute it and/or modify it
  under the terms of the GNU Library General Public License as published by
  the Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public License
  for more details.

  You should have received a copy of the GNU Library General Public License
  along with this library; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

  ---------------------------------------------------------------------------

    Author : Luc DAVID Email: luckylazarus@free.fr
    Copyright (C) 2006

  ---------------------------------------------------------------------------

    20.07.2006 - V0.20
      First public release. For testing and debuging SqlitePassDbo...

    03.09.2006 - V0.21
      Added more database and dataset infos, bugs fixes...

    03.09.2006 - V0.22
      New design especially for datasets infos, indexes infos added, bugs fixes...

    17.10.2006 - V0.23
      Added toolbars
      Reworks and bugs fixes on indexes infos,
      Added index creation dialog,
      Added index rename dialog,
      Added View infos
      Added Database utilities (Compact size, Transactions...etc)

    12/11/2006 - V0.24
      Added Attach - Detach Database support
      Rework on Toolbar design

  --------------------------------------------------------------------------- }


unit MainUnit;

{$IFDEF FPC}
{$MODE Delphi}
{$ENDIF}
interface

uses
{$IFDEF FPC}
LCLIntf,
{$ELSE}
Windows,
{$ENDIF}

Classes, Controls, SqlitePassDbo, SQLitePassDboTest, CreateNewIndex, RenameItem, stdctrls, ComCtrls, SysUtils, db, Dialogs,
DbCtrls, Buttons, ExtCtrls, Graphics, ActnList, Forms;

//const
//DefaultDatabase = 'C:\MyLazarus\SqlitePass\Demo\Databases\Cars.kexi';
//----------------------------------------------------------------
procedure MainForm_DoFormCreate(Self: TMainForm);
procedure MainForm_DoTvChange(Self: TMainForm;  Node: TTreeNode);
procedure MainForm_UpdateStatusBar;
// Database
procedure MainForm_DoActionOpenDbClick(Self: TMainForm; dbName: String);
procedure MainForm_DoActionCloseDbClick;
procedure MainForm_DoShowDatabaseInfos;
procedure MainForm_DoRefreshTreeview;
procedure MainForm_DoRefreshDbInfos;
procedure MainForm_DoDbSettingChanged;
procedure MainForm_DoSbApplyDbChangesClick;
procedure MainForm_DoSbCancelDbChangesClick;
procedure MainForm_DoDatabaseCompactSize;
procedure MainForm_UpdateActionListDatabase(Enable: Boolean);
procedure MainForm_DoActionAttachDatabase;
procedure MainForm_DoActionDetachDatabase;
// Database Transactions
procedure MainForm_DoActionDatabaseTransactionStartExecute;
procedure MainForm_DoActionDatabaseTransactionCommitExecute;
procedure MainForm_DoActionDatabaseTransactionRollbackExecute;
// Datasets
function FieldTypeAsString(FieldType: TFieldType): String;
function BoolAsString(Value: Boolean): String;
procedure MainForm_DoShowDatasetInfos(Node: TTreeNode);
// Datasets indexes
procedure MainForm_DoRefreshIndexList;
procedure MainForm_DoRefreshIndexInfos;
procedure MainForm_DoCreateNewIndex;
procedure MainForm_DoDeleteIndex;
procedure MainForm_DoActionIndexRename;
// Picture
procedure MainForm_DoCbShowGraphicClick;
procedure MainForm_DoBtLoadPictureFromFileClick;
procedure MainForm_DoBtClearPictureClick;
procedure MainForm_DoBtSavePictureToFileClick;
Procedure MainForm_DoComboBoxGraphicFieldsChange;
// Memo
procedure MainForm_DoCbShowMemoClick;
procedure MainForm_DoBtLoadMemoFromFileClick;
procedure MainForm_DoBtClearMemoClick;
procedure MainForm_DoBtSaveMemoToFileClick;
Procedure MainForm_DoComboBoxMemoFieldsChange;
// SQL Direct
procedure MainForm_DoRunSQL;
// Testing
procedure MainForm_DoSbAppendRecordsClick;
// Help - Infos
procedure MainForm_DoShowDemoInfos;
// Setup a simple Toolbar
procedure InitToolBar(ToolBar: TPanel; ActionList: TActionList);
procedure DoUpdateToolBar(ToolBar: TPanel;UpdateAll: Boolean);
procedure MainForm_OnToolBarBtnMouseMove(ToolBar: TPanel; Im: TImage);
procedure MainForm_OnToolBarBtnClick(Im: TImage);
procedure DrawActionBitmap(aBitmap: TBitmap; ImageIndex: Integer; Menu, Enabled: Boolean);
Procedure ResfreshMenu;
{ ----------------------------------------------------------------
                              FormCreateNewIndex
------------------------------------------------------------------ }
procedure FormCreateNewIndex_DoFormCreate;
procedure FormCreateNewIndex_DoOnCbIndexedFieldClick(Sender: TObject);
procedure FormCreateNewIndex_DoOnImIndexDirectionClick(Sender: TObject);
procedure FormCreateNewIndex_DoRefreshIndexDefinition;
function FormCreateNewIndex_DoFindCbIndexedField(Cb: TCheckBox): Integer;
function FormCreateNewIndex_DoFindImIndexDirection(Im: TImage): Integer;
procedure FormCreateNewIndex_DoButtonOkClick(Sender: TObject);
{ ---------------------------------------------------------------- }

implementation


procedure MainForm_DoFormCreate(Self: TMainForm);
var
i: integer;
begin
With MainForm do
begin
CbIndexedFields:= TList.Create;
ImIndexDirection:= TList.Create;
InitToolBar(PanelDatabaseToolBar, ActionListDatabase);
InitToolBar(PanelIndexesToolBar, ActionListIndexes);
InitToolbar(PanelDatasetToolBar, ActionListDataset);
MyDb := TSqlitePassDatabase.Create(Self);
MyDataset := TSqlitePassDataset.Create(Self);
MyDataset.Database := MyDb;
MySQLDirectDataset := TSqlitePassDataset.Create(Self);
MySQLDirectDataset.Database := MyDb;
dsSQLDirect.DataSet := MySQLDirectDataset;
MyDataset.DebugScreen := MemoDebug;
{$IFNDEF FPC}
//MainForm_DoActionOpenDbClick(Self, DefaultDatabase);
{$ENDIF}
MainForm_DoActionCloseDbClick;
MainForm_DoTvChange(MainForm,  nil);
MainForm_DoSbCancelDbChangesClick;
MainForm_UpdateStatusBar;
end;
end;

Procedure ResfreshMenu;
var
i, j : integer;
begin
With MainForm do
begin
For i := 0 to Pred(MainMenu.Items.Count) do
    For j := 0 to Pred(MainMenu.Items[i].Count) do
    begin
    if TAction(MainMenu.Items[i].Items[j].Action) <> nil
       then DrawActionBitmap(MainMenu.Items[i].Items[j].Bitmap, MainMenu.Items[i].Items[j].ImageIndex, True, MainMenu.Items[i].Items[j].Enabled);
    if MainMenu.Items[i].Items[j].Caption <> '-'
       then MainMenu.Items[i].Items[j].Caption := '  '+MainMenu.Items[i].Items[j].Caption;
    end;
end;
end;

{ ToolBar }
procedure InitToolBar(ToolBar: TPanel; ActionList: TActionList);
var
i, VOffset: Integer;
Im: TImage;
begin

with ToolBar do
begin
Height := 30;
Color := cl3DLight;
OnMouseMove := MainForm.OnToolBarMouseMove;
ShowHint := True;
end;
VOffset := 6;
for i := 0 to Pred(ActionList.ActionCount) do
begin
   Im := TImage.Create(ToolBar);
   with Im do
     begin
     Parent := ToolBar;
     AutoSize := False;
     if TAction(ActionList.Actions[i]).Caption = '-'
        then begin { Draw separator }
        Top := 4;
        Height := 22;
        Width := 2;
        Left := VOffset;
        Inc(VOffset, Width + 2);
        Action := ActionList.Actions[i];
        Canvas.Brush.Color := clBtnHighlight;
        Canvas.FillRect(Rect(0,0,1,20));
        Canvas.Brush.Color := clBtnShadow;
        Canvas.FillRect(Rect(1,0,2,20));
        end
        else begin { Draw Action button }
        Top := 4;
        Height := 22;
        Width := 22;
        Left := VOffset;
        Inc(VOffset, Width+2);
        Action := ActionList.Actions[i];
        OnMouseMove := OnToolBarBtnMouseMove;
        OnClick := OnToolBarBtnClick;
        end;
   end;
end;
end;

procedure DrawActionBitmap(aBitmap: TBitmap; ImageIndex: Integer; Menu, Enabled: Boolean);
var
offset: integer;
R: TRect;
begin
if Menu
   then begin
        aBitmap.Height := 16;
        aBitmap.Width := 16;
        offset := 0;
        end
   else begin
        aBitmap.Height := 22;
        aBitmap.Width := 22;
        R := Rect(0,0,aBitmap.Width,aBitmap.Height);
        {$IFDEF FPC}
        aBitmap.Canvas.Frame3d(R, 1, bvLowered);
        {$ELSE}
        Frame3d(aBitmap.Canvas, R, clBtnHighlight,cl3DDkShadow,1);
        R := Rect(1,1,aBitmap.Width-1,aBitmap.Height-1);
        Frame3d(aBitmap.Canvas, R, cl3DDkShadow,clBtnHighlight,1);
        {$ENDIF}
        offset := 3;
        end;
aBitmap.TransparentColor := clFuchsia;
If Enabled
   then MainForm.ImageListEnabled.Draw(aBitmap.Canvas, offset, offset, ImageIndex)
   else MainForm.ImageListDisabled.Draw(aBitmap.Canvas, offset, offset, ImageIndex);

end;

procedure DoUpdateToolBar(ToolBar: TPanel; UpdateAll: Boolean);
var
i: Integer;
ImTemp: TImage;
Act: TAction;
begin
for i := 0 to Pred(ToolBar.ControlCount) do
begin
ImTemp := TImage(ToolBar.Controls[i]);
Act:= TAction(ImTemp.Action);
if (Act <> nil) then
if (ImTemp.Tag = 1) or (UpdateAll and (Act.Caption <> '-')) then
     begin
     DrawActionBitmap(imTemp.Picture.Bitmap, Act.ImageIndex, False, (Act.Enabled and Assigned(Act.OnExecute)));
     ImTemp.Tag := 0;
     end;
end;
end;

procedure MainForm_OnToolBarBtnMouseMove(ToolBar: TPanel; Im: TImage);
begin
DoUpdateToolBar(ToolBar, False);
With Im do
  begin
  canvas.Brush.Color := $00FF8080;
  Canvas.FrameRect(Rect(0,0,Im.Width, Im.Height));
  Tag := 1;
  end;
end;

procedure MainForm_OnToolBarBtnClick(Im: TImage);
begin
With Im do begin
canvas.Brush.Color := clRed;
Canvas.FrameRect(Rect(0,0,Im.Width, Im.Height));
Refresh;
Sleep(60);
canvas.Brush.Color := cl3DLight;
Canvas.FrameRect(Rect(0,0,Im.Width, Im.Height));
Action.Execute;
end;
end;

{ ToolBar - end }

procedure MainForm_DoTvChange(Self: TMainForm;  Node: TTreeNode);
var
i : Integer;
begin
With MainForm do
begin
{ Database is closed ? }
if Node = nil
   then begin
   for i := 0 to Pred(PageControl.PageCount)
    do PageControl.Pages[i].TabVisible := False;
   exit;
   end;

{ Database }
if Node = DatabaseNode
   then begin
   MainForm_DoShowDatabaseInfos;
   PageControl.Pages[0].Tabvisible := True;
   PageControl.Pages[1].Tabvisible := False;
   PageControl.Pages[2].Tabvisible := False;
   end;

if Node.Parent = nil then exit;

{ Datasets }
if ((Node.Parent = TablesNode) or (Node.Parent = SystemTablesNode)
     or (Node.Parent = QueriesNode) or (Node.Parent = ViewsNode))
     then begin
     MainForm_DoShowDatasetInfos(Node);
     PageControl.Pages[0].Tabvisible := False;
     PageControl.Pages[1].Tabvisible := True;
     PageControl.Pages[2].Tabvisible := False;
     PageControlDataset.Pages[2].Tabvisible := (Node.Parent <> QueriesNode)
                                                and (Node.Parent <> ViewsNode);
     end;

{ SQL Execute }
if Node = SQLNode
   then begin
   PageControl.Pages[0].Tabvisible := False;
   PageControl.Pages[1].Tabvisible := False;
   PageControl.Pages[2].Tabvisible := True;
   end;

end;
end;

procedure MainForm_UpdateStatusBar;
var
S: String;
begin
With MainForm do
begin
if MyDb.Connected then
begin
StatusBar.Panels[0].Text := ' Sqlite Engine : ' + MyDb.VersionInfo.SqliteLibrary;
Case MyDb.DatabaseType of
     dbtSqlitePass : S := 'SQLitePass';
     dbtKexi : S := 'Kexi';
     dbtSqliteAdmin: S := 'SQLite Administrator';
     dbtFpc : S := 'Free Pascal Compiler';
     else S := 'Unknown...';
     end;
StatusBar.Panels[1].Text := ' Database : ' + MyDb.Database;
StatusBar.Panels[2].Text := ' Database type : ' + S;
if MyDataset.Active
   then begin
   StatusBar.Panels[3].Text := ' Records count : ' + IntToStr(MyDataset.RecordCount);
   StatusBar.Panels[4].Text := ' Elapsed Time : ' + FloatToStr(MyDataset.Database.Engine.ElapsedTime);
   end;
end
else begin
StatusBar.Panels[0].Text := ' Sqlite Engine : Unknown...';
StatusBar.Panels[1].Text := ' Database : Unknown...';
StatusBar.Panels[2].Text := ' Database type : Unknown...';
StatusBar.Panels[3].Text := ' Records count : Unknown...';
StatusBar.Panels[4].Text := ' Elapsed Time : Unknown...';
end;
end;
end;


{ ------- Database ------- }

procedure MainForm_DoActionOpenDbClick(Self: TMainForm; dbName: String);
begin
With MainForm do
begin
if (dbName = '')
   then if OpenFileDialog.Execute
           then dbName := OpenFileDialog.FileName;
if dbName = '' then exit;
MyDb.Close;
MyDb.Database := dbName;
MyDb.Open;
MainForm_DoRefreshTreeview;
end;
end;

procedure MainForm_DoRefreshTreeview;
var
i: Integer;
TvItem: TTreeNode;
begin
With MainForm do
begin
Tv.Items.Clear;
DatabaseNode:= Tv.Items.Add(nil,'Database');
TablesNode  := Tv.Items.AddChild(DatabaseNode,'Tables');
TablesNode.ImageIndex := 2;
QueriesNode := Tv.Items.AddChild(DatabaseNode,'Queries');
ViewsNode    := Tv.Items.AddChild(DatabaseNode,'Views');
SystemTablesNode := Tv.Items.AddChild(DatabaseNode,'System Tables');
SQLNode := Tv.Items.AddChild(DatabaseNode,'SQL Execute');

for i:= 0 to Pred(MyDb.TableDefs.Count)
    do if Not MyDb.IsSystemTable(MyDb.TableDefs[i].TableName)
       then begin
            TvItem := Tv.Items.AddChild(TablesNode,MyDb.TableDefs[i].TableNameExt);
            TvItem.ImageIndex := -1;
            end
            else Tv.Items.AddChild(SystemTablesNode,MyDb.TableDefs[i].TableNameExt);

for i:= 0 to Pred(MyDb.QueryDefs.Count)
    do Tv.Items.AddChild(QueriesNode,MyDb.QueryDefs[i].QueryNameExt);

for i:= 0 to Pred(MyDb.Views.Count)
    do Tv.Items.AddChild(ViewsNode,MyDb.Views[i].ViewName);

Tv.FullExpand;
Tv.Selected := DatabaseNode;
MainForm_UpdateActionListDatabase(True);
MainForm_UpdateStatusBar;
end;
end;

procedure MainForm_DoActionCloseDbClick;
begin
With MainForm do
begin
MyDb.Close;
Tv.Items.Clear;
MainForm_DoTvChange(MainForm,  nil);
MainForm_UpdateActionListDatabase(False);
MainForm_UpdateStatusBar;
end;
end;

{ ------ Database Infos ------ }
procedure MainForm_DoShowDatabaseInfos;
begin
{
 Property DatabaseType         : TSqlitePassDatabaseType  Read FDatabaseType   Write SetFDatabaseType;

 Property Options              : TSqlitePassDatabaseOptions Read FOptions            Write FOptions;
         Property AutoVacuum: Boolean Read GetFAutoVacuum Write SetFAutoVacuum;
         Property CacheSize: Integer Read GetFCacheSize Write SetFCacheSize;
         Property DefaultCacheSize: Integer Read GetFDefaultCacheSize Write SetFDefaultCacheSize;
         Property CaseSensitiveLike: Boolean Read FCaseSensitiveLike Write SetFCaseSensitiveLike;
         Property CountChanges: Boolean Read GetFCountChanges Write SetFCountChanges;
         Property EmptyResultCallback: Boolean Read GetFEmptyResultCallBack Write SetFEmptyResultCallBack;
         Property Encoding: TSqlitePassEncoding Read GetFEncoding Write SetFEncoding;
         Property FullColumnNames: Boolean Read GetFFullColumnNames Write SetFFullColumnNames; //Think to Set Also ShortColumnNames
         Property PageSize: TSqlitePassPageSize Read GetFPageSize Write SetFPageSize;
         Property Synchronous : TSqlitePassSynchronous Read GetFSynchronous Write SetFSynchronous;
         Property TemporaryStorage: TSqlitePassTempStore Read GetFTemporaryStorage Write SetFTemporaryStorage;

 Property QueryTimeout         : Integer Read FQueryTimeout        Write FQueryTimeout;
 Property ReadOnly             : Boolean Read FReadOnly            Write SetFReadOnly;
 Property ShowSystemObjects    : Boolean Read FShowSysObjects      Write SetFShowSysObjects;
 Property SmartOpen            : Boolean Read FSmartOpen           Write FSmartOpen;
 Property UserTag              : Integer Read FUser                Write SetFUser;

 Property VersionInfo          : TSqlitePassVersionInfo  Read FVersionInfo   Write FVersionInfo;
         Property Component     : String  Read FComponent       Write SetFComponent;
         Property Database      : Integer Read GetFSchema       Write SetFSchema;
         Property Kexi          : String  Read FKexi            Write SetFKexi;
         Property SqliteLibrary : String  Read GetFSqliteEngine Write SetFSqliteEngine;
         Property UserTag       : Integer Read GetFUserTag      Write SetFUserTag;
 }

MainForm_DoRefreshDbInfos;
end;

procedure MainForm_DoRefreshDbInfos;
var
i: integer;
f: TFileStream;
begin
With MainForm do
begin
if not Mydb.Connected then exit;
MyDb.RefreshDefinitions;
Tv.Selected := DatabaseNode;

{ DatabaseType }
Case MyDb.DatabaseType of
     dbtSqlitePass : LabelDbType.Caption := 'SQLitePass';
     dbtKexi       : LabelDbType.Caption := 'Kexi';
     dbtSqliteAdmin: LabelDbType.Caption := 'SQLite Administrator';
     dbtFpc        : LabelDbType.Caption := 'Free Pascal Compiler';
     else LabelDbType.Caption := 'Unknown...';
     end;

{ file size }
f := TFileStream.Create(MyDb.Database, fmOpenRead or fmShareDenyNone);;
LabelDbFileSize.Caption := IntToStr(F.Size) + ' octets';
{ Cache Size }
EditDbCacheSize.Text := IntToStr(MyDb.Options.CacheSize);
{ Query Timeout }
EditDbQueryTimeout.Text := IntToStr(MyDb.QueryTimeout);
{ User Tag }
EditDbUserTag.Text := IntToStr(MyDb.UserTag);
{ Read Only }
CbDbReadOnly.Checked := MyDb.ReadOnly;
{ Show System Objects }
CbDbShowSystemObjects.Checked := MyDb.ShowSystemObjects;
{ Smart Open }
CbDbSmartOpen.Checked := MyDb.SmartOpen;
{ Auto Vacuum }
CbDbAutoVacuum.Checked := MyDb.Options.AutoVacuum;
{ Count Changes }
CbDbCountChanges.Checked := MyDb.Options.CountChanges;
{ Case Sensitive "Like" }
CbDbCaseSensitiveLike.Checked := MyDb.Options.CaseSensitiveLike;
{ Empty Result Callback }
CbDbEmptyResultCallback.Checked := MyDb.Options.EmptyResultCallback;
{ Full Column Name }
CbDbFullColumnName.Checked := MyDb.Options.FullColumnNames;
{ Page Size }
EditDbCacheSize.Text := IntToStr(MyDb.Options.PageSize);
{ Encoding }

// TODO - add TComboListBox
EditDbEncoding.Text := '';
{ Temporary Storage }
EditDbTemporaryStorage.Text := '';
{ Synchronous }

{ ComponentVersion }
LabelDbComponentVersion.Caption := MyDb.VersionInfo.Component;
{ Sqlite Library Version }
LabelDbSqliteLibVersion.Caption := MyDb.VersionInfo.SqliteLibrary;
{ Database Version }
LabelDbVersion.Caption := IntToStr(MyDb.VersionInfo.Database);
{ Database created with Kexi version }
LabelDbKexiVersion.Caption := MyDb.VersionInfo.Kexi;

{ --- Display Attached Databases Infos ---  }
With GridAttachedDatabases do
  begin
  cells[0,0] := 'Alias name';
  cells[1,0] := 'Database';
  { Clear Grid }
  for i := 1 to 9 do  // should be 1 to 10 -> Lazarus TVirtualGrid.GetCells(Col, Row: Integer) bug ?
    begin
    Cells[0,i] := '';
    Cells[1,i] := '';
    end;
  { Update Grid }
  if (MyDb.Databases.count > 0) then
  begin
  for i := 0 to Pred(MyDb.Databases.count) do
    begin
    Cells[0,i+1] := MyDb.Databases[i].Name;
    Cells[1,i+1] := MyDb.Databases[i].Database;
    end;
  end;
  end;
  ActionDatabaseDetach.Enabled := (GridAttachedDatabases.Cells[0,GridAttachedDatabases.row] <> '');
  DoUpdateToolBar(PanelDatabaseToolBar,True);
end;
end;

procedure MainForm_DoDbSettingChanged;
begin
With MainForm do
begin
// TODO
(*SbDbApplyChanges.Enabled := True;
SbDbCancelChanges.Enabled := True;*)
end;
end;

procedure MainForm_DoSbApplyDbChangesClick;
begin
With MainForm do
begin
// TODO
(*SbDbApplyChanges.Enabled := False;
SbDbCancelChanges.Enabled := False;*)
end;
end;

procedure MainForm_DoSbCancelDbChangesClick;
begin
With MainForm do
begin
//TODO
(*SbDbApplyChanges.Enabled := False;
SbDbCancelChanges.Enabled := False;*)
end;
end;

procedure MainForm_DoDatabaseCompactSize;
begin
With MainForm do
begin
ShowMessage(IntToStr(Mydb.Compact) + ' octets have been freed from the database : '+ MyDb.Database);
end;
end;

procedure MainForm_UpdateActionListDatabase(Enable: Boolean);
var
i: integer;
begin
With MainForm do
begin
for i := 0 to Pred(ActionListDatabase.ActionCount) do
    TAction(ActionListDatabase.Actions[i]).Enabled := Enable;

ActionDatabaseOpen.Enabled := Not Enable;
ActionDatabaseTransactionCommit.Enabled := False;
ActionDatabaseTransactionRollback.Enabled := False;
ActionDatabaseDetach.Enabled := False;
DoUpdateToolBar(PanelDatabaseToolBar,True);
end;
end;

procedure MainForm_DoActionAttachDatabase;
var
AttachedDb: String;
begin
With MainForm do
begin
if OpenFileDialog.Execute
   then MyDb.AttachDatabase(OpenFileDialog.FileName,'');
MainForm_DoRefreshTreeview;
MainForm_DoRefreshDbInfos
end;
end;

procedure MainForm_DoActionDetachDatabase;
begin
With MainForm do
begin
MyDb.DetachDatabase(GridAttachedDatabases.Cells[0,GridAttachedDatabases.row]);
MainForm_DoRefreshTreeview;
MainForm_DoRefreshDbInfos;
end;
end;

{ ------ Database Transactions ------ }

procedure MainForm_DoActionDatabaseTransactionStartExecute;
begin
With MainForm do
begin
MyDb.Engine.Transaction.Start;
ActionDatabaseTransactionStart.Enabled := False;
ActionDatabaseTransactionCommit.Enabled := True;
ActionDatabaseTransactionRollback.Enabled := True;
DoUpdateToolBar(PanelDatabaseToolBar,True);
end;
end;

procedure MainForm_DoActionDatabaseTransactionCommitExecute;
begin
With MainForm do
begin
MyDb.Engine.Transaction.Commit;
ActionDatabaseTransactionStart.Enabled := True;
ActionDatabaseTransactionCommit.Enabled := False;
ActionDatabaseTransactionRollback.Enabled := False;
DoUpdateToolBar(PanelDatabaseToolBar,True);
end;
end;

procedure MainForm_DoActionDatabaseTransactionRollbackExecute;
begin
With MainForm do
begin
MyDb.Engine.Transaction.Rollback;
ActionDatabaseTransactionStart.Enabled := True;
ActionDatabaseTransactionCommit.Enabled := False;
ActionDatabaseTransactionRollback.Enabled := False;
DoUpdateToolBar(PanelDatabaseToolBar,True);
end;
end;


{ ------- Datasets ------- }

procedure MainForm_DoCbShowMemoClick;
begin
With MainForm do
begin
if cbShowMemo.Checked
   then MainForm_DoComboBoxMemoFieldsChange;
end;
end;

{ ------- DbPicture and Blob ------- }

procedure MainForm_DoCbShowGraphicClick;
begin
With MainForm do
begin
if cbShowGraphic.Checked
 then MainForm_DoComboBoxGraphicFieldsChange;
end;
end;

Procedure MainForm_DoBtLoadPictureFromFileClick;
begin
With MainForm do
begin
if OpenPictureDialog.Execute
   then begin
        MyDataset.Edit;
        TBlobField(DbImage.Field).LoadFromFile(OpenPictureDialog.Filename);
        end;
end;
end;

procedure MainForm_DoBtClearPictureClick;
begin
With MainForm do
begin
 MyDataset.Edit;
 DbImage.Field.Clear;
end;
end;

procedure MainForm_DoBtSavePictureToFileClick;
begin
With MainForm do
begin
If SavePictureDialog.Execute
   then TBlobField(DbImage.Field).SaveToFile(SavePictureDialog.Filename);
end;
end;

Procedure MainForm_DoComboBoxGraphicFieldsChange;
var
F: TField;
begin
With MainForm do
begin
if ComboBoxGraphicFields.Text = '' then exit;
F := MyDataset.FieldByName(ComboBoxGraphicFields.Text);
if (F <> nil) and CbShowGraphic.Checked
   then begin
        if F.DataType = ftBlob
           then DbImage.DataField := ComboBoxGraphicFields.Text;
        end
        else DbImage.DataField := '';
end;
end;

{ ------- DbMemo and Blob ------- }
procedure MainForm_DoBtLoadMemoFromFileClick;
begin
With MainForm do
begin
if OpenFileDialog.Execute
   then begin
        MyDataset.Edit;
        TMemoField(DbMemo.Field).LoadFromFile(OpenFileDialog.FileName);
        end;
end;
end;

procedure MainForm_DoBtClearMemoClick;
begin
With MainForm do
begin
 MyDataset.Edit;
 TBlobField(DbMemo.Field).Clear;
 MyDataset.Post;
end;
end;

procedure MainForm_DoBtSaveMemoToFileClick;
begin
With MainForm do
begin
If SaveFileDialog.Execute
   then TBlobField(DbMemo.Field).SaveToFile(SaveFileDialog.Filename);
end;
end;

Procedure MainForm_DoComboBoxMemoFieldsChange;
var
F: TField;
begin
With MainForm do
begin
if ComboBoxMemoFields.Text = '' then exit;
F := MyDataset.FieldByName(ComboBoxMemoFields.Text);
if (F <> nil) and CbShowMemo.Checked
   then begin
        if F.DataType = ftMemo
           then DbMemo.DataField := ComboBoxMemoFields.Text;
        end
        else DbMemo.DataField := '';
end;
end;



procedure MainForm_DoShowDatasetInfos(Node: TTreeNode);
var
i: Integer;
ListItem: TListItem;
MyTableDef: TSqlitePassTableDef;
begin
With MainForm do
begin
  PageControl.ActivePage := TsDataset;
  //
  DbImage.DataField := '';
  ComboBoxGraphicFields.Clear;
  ComboBoxGraphicFields.Text := '';
  DbMemo.DataField := '';
  ComboBoxMemoFields.Clear;
  ComboBoxMemoFields.Text := '';
  //
  With MyDataset do
  begin
  Close;
  { For test purpose }
  //RecordLowerLimit := 2;
  //RecordUpperLimit := 3;
  { }
  Database := MyDb;
  DatasetName := Node.Text;
  Open;
  PageControl.ActivePage.Caption := 'Dataset : ' + DatasetName;
  MemoSqlCreateStmt.Clear;
  MyTableDef := MyDb.TableDefs.FindTable(Node.Text);
  If Assigned(MyTableDef)
     then MemoSqlCreateStmt.Lines.Text := MyTableDef.Sql;
  MemoSqlSelectStmt.Clear;
  MemoSqlSelectStmt.Lines.Text := SQL.Text;
  if ReadOnly or not CanModify
     then begin
     DbNavDataset.VisibleButtons := [nbFirst, nbPrior, nbNext, nbLast, nbRefresh];
     ImageDatasetReadOnly.Visible := True;
     LabelDatasetReadOnly.Visible := True;
     end
     else begin
     DbNavDataset.VisibleButtons := [nbFirst, nbPrior, nbNext, nbLast, nbInsert, nbEdit, nbDelete, nbPost, nbCancel, nbRefresh];
     ImageDatasetReadOnly.Visible := False;
     LabelDatasetReadOnly.Visible := False;
     end;
  First;
  end;
  //
  DataSource.DataSet := MyDataset;
 // DbGridDatasetData.Visible := Not MyDataset.IsEmpty;

  { --- Display FieldDefs Infos ---  }
  With GridFieldDefs do
  begin
  RowCount := MyDataset.Fields.count+1;
  ColCount := 8;
  cells[0,0] := 'Name';
  cells[1,0] := 'Display Name';
  cells[2,0] := 'Type';
  cells[3,0] := 'Size';
  cells[4,0] := 'Precision';
  cells[5,0] := 'Required';
  cells[6,0] := 'Read Only';
  cells[7,0] := 'Can Modify';

  For i := 0 to Pred(MyDataset.Fields.count) do
  begin
  Cells[0,i+1] := MyDataset.Fields[i].FieldName;
  Cells[1,i+1] := MyDataset.Fields[i].DisplayName;
  Cells[2,i+1] := FieldTypeAsString(MyDataset.Fields[i].DataType);
  Cells[3,i+1] := IntToStr(MyDataset.Fields[i].Size);
//  Cells[4,i+1] := IntToStr(MyDataset.Fields[i].Precision);
  Cells[5,i+1] := BoolAsString(MyDataset.Fields[i].Required);
  Cells[6,i+1] := BoolAsString(MyDataset.Fields[i].ReadOnly);
  Cells[7,i+1] := BoolAsString(MyDataset.Fields[i].CanModify);

  if MyDataset.Fields[i].DataType in  [ftGraphic, ftBlob]
     then ComboBoxGraphicFields.Items.Add(MyDataset.Fields[i].FieldName);
  if MyDataset.Fields[i].DataType in  [ftMemo]
     then ComboBoxMemoFields.Items.Add(MyDataset.Fields[i].FieldName);

  end;
  end;

  { Display IndexDefs Infos }
  MainForm_DoRefreshIndexList;

  { Display available Blobs fields }
  if ComboBoxGraphicFields.Items.Count > 0
     then begin
     ComboBoxGraphicFields.Text := ComboBoxGraphicFields.Items[0];
     MainForm_DoComboBoxGraphicFieldsChange;
     end;

  if ComboBoxMemoFields.Items.Count > 0
     then begin
     ComboBoxMemoFields.Text := ComboBoxMemoFields.Items[0];
     MainForm_DoComboBoxMemoFieldsChange;
     end;
  //
  MyDataset.First;
  MainForm_UpdateStatusBar;
  end;
end;

{ --- Display IndexDefs Infos ---  }
procedure MainForm_DoRefreshIndexList;
var
i: integer;
begin
With MainForm do
begin
MyDataset.IndexDefs.Refresh;
ListBoxIndexDefs.Clear;
For i := 0 to Pred(MyDataset.IndexDefs.count) do
ListBoxIndexDefs.items.Add(MyDataset.IndexDefs[i].IndexName);
if ListBoxIndexDefs.Items.Count > 0
   then ListBoxIndexDefs.ItemIndex := 0;
{ Indexes fields update }
MainForm_DoRefreshIndexInfos;
end;
end;

{ Display current index infos }
procedure MainForm_DoRefreshIndexInfos;
var
i, j: Integer;
MyIndexDef: TSqlitePassIndex;
MyIndexCol: TSqlitePassIndexColumn;
aCheckBox: TCheckBox;
aImage: TImage;
begin
With MainForm do
begin
MemoSQLIndexCreateStmt.Clear;
for i := Pred(ScrollBoxIndexedFields.ControlCount) downto 0
do ScrollBoxIndexedFields.Controls[i].Free;

If ListBoxIndexDefs.Items.Count = 0
   then exit;
{ Show SQL Create statement for the selected index }
MyIndexDef := MyDb.IndexDefs.FindIndex(ListBoxIndexDefs.Items[ListBoxIndexDefs.ItemIndex]);
If not Assigned(MyIndexDef)
   then Exit;
MemoSqlIndexCreateStmt.Lines.Text := MyIndexDef.Sql;
{ Show fields used by the selected index and sort order }
j := 0;
For i := 0 to Pred(MyDataset.FieldCount) do
    Begin
     if (MyDataset.Fields[i].FieldKind=fkData) and (MyDataset.Fields[i].Visible)
        then begin
        MyIndexCol := MyIndexDef.Columns.FindColumn(MyDataset.Fields[i].FieldName);
        aCheckBox := TCheckBox.Create(ScrollBoxIndexedFields);
        With aCheckBox do
             begin
             Parent := ScrollBoxIndexedFields;
             Caption:=MyDataset.Fields[i].FieldName;
             Top  := (j*20)+5;
             Left := 30;
             Width := 200;
             Height := 16;
             Checked := (MyIndexCol <> nil);
             //OnClick:=OnCbIndexedFieldClick;
             end;

        aImage := TImage.Create(ScrollBoxIndexedFields);
        With aImage do
             begin
             Parent := ScrollBoxIndexedFields;
             Top  := (j*20)+4;
             Left := 6;
             Width := 16;
             Height := 16;
             Transparent := True;
             if MyIndexCol <> nil
                then begin
                     Case MyIndexCol.Direction of
                     cidAscending  : begin
                                     Tag := 1;
                                     Picture.Assign(ImageIndexDirectionAsc.Picture);
                                     end;
                     cidDescending : begin
                                     Tag := 2;
                                     Picture.Assign(ImageIndexDirectionDesc.Picture);
                                     end;
                     cidUnknown    : begin
                                     Tag := 3;
                                     Picture.Assign(ImageIndexDirectionUnknown.Picture);
                                     end;
                     end;
                     Visible := True;
                     end else Visible := False;
             end;
        inc(j);
        CbIndexedFields.Add(aCheckBox);
        ImIndexDirection.Add(aImage);
       end;
    end;

CheckBoxUniqueIndex.Checked := MyIndexDef.Unique;
end;
end;

{ Create New index }
procedure MainForm_DoCreateNewIndex;
begin
With MainForm do
begin
FormCreateNewIndex := TFormCreateNewIndex.Create(MainForm);
if FormCreateNewIndex.ShowModal = mrOk
   then MainForm_DoRefreshIndexList;
FormCreateNewIndex.Free;
end;
end;



{ Delete selected index }
procedure MainForm_DoDeleteIndex;
begin
With MainForm do
begin
MyDataset.IndexDefs.DeleteIndex(ListBoxIndexDefs.Items[ListBoxIndexDefs.ItemIndex]);
MainForm_DoRefreshIndexList;
end;
end;

procedure MainForm_DoActionIndexRename;
begin
FormRenameItem := TFormRenameItem.Create(MainForm);
FormRenameItem.LabelRenameItem.Caption := 'Rename index '
                               + MainForm.ListBoxIndexDefs.Items[MainForm.ListBoxIndexDefs.ItemIndex] + ' as :';
if FormRenameItem.ShowModal = mrOk
   then begin
   MainForm.MyDataset.IndexDefs.RenameIndex(
            MainForm.ListBoxIndexDefs.Items[MainForm.ListBoxIndexDefs.ItemIndex], FormRenameItem.EditNewName.Text);
   MainForm_DoRefreshIndexList;
   end;
FormRenameItem.Free;
end;

procedure MainForm_DoRunSQL;
var
SqlText: String;
begin
With MainForm do
begin
SqlText := Lowercase(MemoSqlStmt.Lines.Text);

if Pos('select ', SqlText) = 0
   then MyDb.Engine.Exec(SqlText)
   else begin
        MySQLDirectDataset.Close;
        MySQLDirectDataset.SQL := MemoSqlStmt.Lines;
        MySQLDirectDataset.Open;

        { necessary to refresh the dbgrid -> SqliPassDataset bug ? }
        MySQLDirectDataset.Last;
        MySQLDirectDataset.First;

        end;
end;
end;

{ ------- Testing ------- }
procedure MainForm_DoSbAppendRecordsClick;
var
i, j, TickCount, RecordCount: Integer;
begin
// Inserting Records - Speed Test.
With MainForm do
begin
MyDataset.DisableControls;
TickCount := Integer(GetTickCount);
RecordCount := StrToInt(EditRecordCount.Text);
if CheckBoxUseTransaction.Checked
   then MyDataset.Database.Engine.Transaction.Start;

for i := 0 to RecordCount do
begin
MyDataset.Append;
for j := 0 to Pred(MyDataset.Fields.count) do
Case MyDataset.Fields[j].DataType of
     ftString : MyDataset.Fields[j].Value := 'Enre num : ' + IntToStr(i) + '-T' + IntToStr(TickCount);
     ftInteger: MyDataset.Fields[j].Value := TickCount + j;
     ftFloat  : MyDataset.Fields[j].Value := TickCount + j + 0.57;
     end;
MyDataset.Post;
end;
if CheckBoxUseTransaction.Checked
   then MyDataset.Database.Engine.Transaction.Commit;
MyDataset.EnableControls;
MainForm_UpdateStatusBar;
end;
end;

procedure MainForm_DoShowDemoInfos;
begin
With MainForm do
begin
ShowMessage('SQLiteToolBox. SQLitePass Components Demo Program. Version : '+ MyDb.VersionInfo.Component);
end;
end;

{ ----------------------------------------------------------------------------
                              FormCreateNewIndex
  ---------------------------------------------------------------------------- }

procedure FormCreateNewIndex_DoFormCreate;
begin
FormCreateNewIndex_DoRefreshIndexDefinition;
end;

procedure FormCreateNewIndex_DoOnCbIndexedFieldClick(Sender: TObject);
var
i : Integer;
begin
With FormCreateNewIndex do
begin
  i := FormCreateNewIndex_DoFindCbIndexedField(Sender as TCheckBox);
  if i =-1 then Exit;
  if ACbIndexedFields[i].Checked
     then begin
          AImIndexDirection[i].Picture.Assign(ImageAscending.Picture);
          AImIndexDirection[i].Tag := 1;
          AImIndexDirection[i].Visible := True;
          end
     else AImIndexDirection[i].Visible := False;
end;
end;

procedure FormCreateNewIndex_DoOnImIndexDirectionClick(Sender: TObject);
var
i : Integer;
begin
With FormCreateNewIndex do
begin
  i := FormCreateNewIndex_DoFindImIndexDirection(Sender as TImage);
  if i =-1 then Exit;
  if AImIndexDirection[i].Tag = 1
     then AImIndexDirection[i].Tag := 2
     else AImIndexDirection[i].Tag := 1;

  if AImIndexDirection[i].Tag = 1
     then AImIndexDirection[i].Picture.Assign(ImageAscending.Picture)
     else AImIndexDirection[i].Picture.Assign(ImageDescending.Picture);
end;
end;

procedure FormCreateNewIndex_DoRefreshIndexDefinition;
var
i, j: integer;
MyIndexDef: TSqlitePassIndex;
MyIndexCol: TSqlitePassIndexColumn;
begin
j := 0;
if not assigned(FormCreateNewIndex) then exit;
With FormCreateNewIndex do
begin
For i:= 0 to Pred(MainForm.MyDb.TableDefs.Count)
    do if Not MainForm.MyDb.IsSystemTable(MainForm.MyDb.TableDefs[i].TableName)
       then ComboBoxTableList.Items.Add(MainForm.MyDb.TableDefs[i].TableName);
ComboBoxTableList.ItemIndex := ComboBoxTableList.Items.IndexOf(MainForm.MyDataset.DatasetName);
ComboBoxTableList.Enabled := False;
For i := 0 to Pred(MainForm.MyDataset.FieldCount) do
    Begin
     if (MainForm.MyDataset.Fields[i].FieldKind=fkData) and (MainForm.MyDataset.Fields[i].Visible)
        then begin
        MyIndexCol := TSqlitePassIndexColumn.Create;
        FormCreateNewIndex.ACbIndexedFields[j]:=TCheckBox.Create(FormCreateNewIndex.ScrollBoxIndexedFields);
        With FormCreateNewIndex.ACbIndexedFields[j] do
             begin
             Parent := FormCreateNewIndex.ScrollBoxIndexedFields;
             Caption:=MainForm.MyDataset.Fields[i].FieldName;
             Top  := (j*20)+5;
             Left := 56;
             Width := 200;
             Height := 16;
             OnClick := OnCbIndexedFieldClick;
             end;
        AImIndexDirection[j]:=TImage.Create(ScrollBoxIndexedFields);
        With AImIndexDirection[j] do
             begin
             Parent := FormCreateNewIndex.ScrollBoxIndexedFields;
             Top  := (j*20)+4;
             Left := 32;
             Width := 16;
             Height := 16;
             Transparent := True;
             OnClick := OnImIndexDirectionClick;
             end;
        Inc(j);
       end;
    end;

CheckBoxUniqueIndex.Checked := MyIndexDef.Unique;
end;
end;

Function  FormCreateNewIndex_DoFindCbIndexedField(Cb : TCheckBox):Integer;
var
 i : Integer;
begin
With FormCreateNewIndex do
begin
 Result := -1;
 for i := 0 to 99 do
     begin
       if ACbIndexedFields[i]=Cb then
          begin
            Result := i;
            Exit;
          end;
     end;
end;
end;

Function FormCreateNewIndex_DoFindImIndexDirection(Im : TImage):Integer;
var
 i : Integer;
begin
With FormCreateNewIndex do
begin
 Result := -1;
 for i := 0 to 99 do
     begin
       if AImIndexDirection[i]=Im then
          begin
            Result := i;
            Exit;
          end;
     end;
end;
end;

procedure FormCreateNewIndex_DoButtonOkClick(Sender: TObject);
var
i: integer;
FieldSelected: Boolean;
MyIndexDef: TSqlitePassDatasetIndex;
MyIndexColumn: TSqlitePassIndexColumn;
begin
With FormCreateNewIndex do
begin
If EditIndexName.Text = ''
   then begin
   MessageDlg('The index name is missing.', mtinformation, [mbOk], 0);
   EditIndexName.SetFocus;
   ModalResult := mrNone;
   Abort;
   end;

FieldSelected := False;
for i := 0 to Pred(MainForm.MyDataset.FieldCount) do
begin
if ACbIndexedFields[i].Checked
   then begin
   FieldSelected := True;
   break;
   end;
end;

If not FieldSelected
   then begin
   MessageDlg('No field selected for this index', mtinformation, [mbOk], 0);
   ModalResult := mrNone;
   Abort;
   end;
Try
MyIndexDef := TSQLitePassDatasetIndex.Create(MainForm.MyDataset.IndexDefs);
MyIndexDef.TableName := ComboBoxTableList.Text;
MyIndexDef.IndexName := EditIndexName.Text;
MyIndexDef.Unique := CheckBoxUniqueIndex.Checked;

for i := 0 to Pred(MainForm.MyDataset.FieldCount) do
begin
if ACbIndexedFields[i].Checked
   then begin
   MyIndexColumn := TSqlitePassIndexColumn.Create;
   MyIndexColumn.ColumnName := ACbIndexedFields[i].Caption;
   if AImIndexDirection[i].Tag = 1
      then MyIndexColumn.Direction := cidAscending
      else MyIndexColumn.Direction := cidDescending;
   MyIndexDef.Columns.Add(MyIndexColumn);
   end;
end;
MainForm.MyDataset.IndexDefs.CreateIndex(MyIndexDef);
except
ModalResult := mrNone;
raise;
end;
end;
end;

end.

