{   This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  ---------------------------------------------------------------------------

    SqlitePass Database Objects Package
    Registration unit
    Author : Luc DAVID Email: luckylazarus@free.fr
    Copyright (C) 2006-2008

  ---------------------------------------------------------------------------

    Major changes are indicated in the \Documentation\Changes.pdf file
    Last update 25.02.2008

  --------------------------------------------------------------------------- }


unit RegisterSqlitePassDbo;

{$i SqlitePassDbo.inc}

{$IFNDEF FPC}
{$R ../Dx_SqlitePassDbo_Designtime.dcr}
{$ENDIF}
interface

uses

{$IFDEF FPC}
 LResources, componenteditors, PropEdits, LazarusPackageIntf, Buttons,
{$ELSE}
{$IFNDEF Delphi6}DsgnIntf,{$ELSE} DesignIntf, DesignEditors,{$ENDIF}
{$ENDIF}
 Classes, SysUtils, Forms, Controls, ComCtrls, StdCtrls, ExtCtrls, Dialogs,
 Db,
 SqlitePassDbo, SqlitePassDesignErrorLang,
 SqlitePassSortByDialog, SqlitePassChooseDatasetDialog, SqlitePassIndexesDialog,
 SqlitePassMasterDetailFieldsDialog, SqlitePassFieldDefsDialog, SqlitePassDataTypesDialog,
 SqlitePassCustomFieldDefsDialog, SqlitePassFiltersDialog;

Type

{ Database Editors }

TDatabaseFileNameEditor = class(TStringProperty)
  Public
    Procedure Edit;override;
    Function  GetAttributes: TPropertyAttributes; override;
  End;

TDatabaseSQLiteLibraryEditor = class(TStringProperty)
  Public
    Procedure Edit;override;
    Function  GetAttributes: TPropertyAttributes; override;
  End;

TDatabaseDatatypesEditor = class(TStringProperty)
  Public
    Procedure Edit;override;
    Function  GetAttributes: TPropertyAttributes; override;
  End;

TDatabaseCustomFieldDefsEditor = class(TStringProperty)
    Procedure Edit;override;
    Function  GetAttributes: TPropertyAttributes; override;
  end;

TDatabaseDatatypesComponentEditor = class(TComponentEditor)
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;


{ Dataset Editors }

TDatasetNameEditor = class(TStringProperty)
  Public
    Procedure Edit;override;
    Function  GetAttributes: TPropertyAttributes; override;
  End;

TDatasetSortedByEditor = class(TStringProperty)
  Public
    Procedure Edit;override;
    Function  GetAttributes: TPropertyAttributes; override;
  End;

TDatasetFieldDefsEditor = class(TStringProperty)
  Public
    Procedure Edit;override;
    Function  GetAttributes: TPropertyAttributes; override;
  End;

TDatasetFilterEditor = class(TStringProperty)
  Public
    Procedure Edit;override;
    Function  GetAttributes: TPropertyAttributes; override;
  End;

TDatasetIndexesEditor = class(TStringProperty)
  Public
    Procedure Edit;override;
    Function  GetAttributes: TPropertyAttributes; override;
  End;

TDatasetMasterDetailFieldsEditor = class(TStringProperty)
  Public
    Procedure Edit;override;
    Function  GetAttributes: TPropertyAttributes; override;
  End;

var
LastDatabaseDir: String;

procedure Register;


implementation
procedure Register;
Begin
 { TSqlitePassDatabase }
 RegisterComponents('SqlitePassDbo', [TSqlitePassDatabase]);
 RegisterComponentEditor(TSqlitePassDatabase, TDatabaseDatatypesComponentEditor);
 RegisterPropertyEditor(TypeInfo(String),TSqlitePassDatabase,'Database',TDatabaseFileNameEditor);
 RegisterPropertyEditor(TypeInfo(String),TSqlitePassDatabase,'SQLiteLibrary',TDatabaseSQLiteLibraryEditor);
 RegisterPropertyEditor(TypeInfo(TSqlitePassFieldTypesTranslationRules),TSqlitePassDatabaseDataTypeOptions,'TranslationRules',TDatabaseDatatypesEditor);
 RegisterPropertyEditor(TypeInfo(TSqlitePassCustomFieldDefs),TSqlitePassDatabaseDataTypeOptions,'CustomFieldDefs',TDatabaseCustomFieldDefsEditor);

 { TSqlitePassDataset }
 RegisterComponents('SqlitePassDbo', [TSqlitePassDataset]);
 RegisterPropertyEditor(TypeInfo(String),TSqlitePassDataset,'DatasetName',TDatasetNameEditor);
 RegisterPropertyEditor(TypeInfo(String),TSqlitePassDataset,'MasterFields',TDatasetMasterDetailFieldsEditor);
 RegisterPropertyEditor(TypeInfo(String),TSqlitePassDataset,'SortedBy',TDatasetSortedByEditor);
 RegisterPropertyEditor(TypeInfo(TSqlitePassDatasetIndexDefs),TSqlitePassDataset,'IndexDefs',TDatasetIndexesEditor);
 RegisterPropertyEditor(TypeInfo(String),TSqlitePassDataset,'FieldDefsInfo',TDatasetFieldDefsEditor);
 RegisterPropertyEditor(TypeInfo(String),TSqlitePassDataset,'Filter',TDatasetFilterEditor);
End;

{ TDatasetNameEditor }

Function TDatasetNameEditor.GetAttributes: TPropertyAttributes;
begin
   Result:= [paDialog];
end;

procedure TDatasetNameEditor.Edit;
var
F: TSqlitePassChooseDatasetDialog;
Dataset: TSqlitePassDataset;
IsOpen: Boolean;
begin
Dataset := TSqlitePassDataset(GetComponent(0));

if Assigned(Dataset.Database) then
   begin
   F := TSqlitePassChooseDatasetDialog.Create(Nil, Dataset);
   if (F.ShowModal = mrOk)
      then begin
           IsOpen := Dataset.Active;
           Dataset.Close;
           SetStrValue(F.Tv.Selected.Text);
           If IsOpen then Dataset.Active := True;
           end;
   F.Free;
   end
   else MessageDlg(Msg1010, mtInformation, [mbOk], 0);
End;

{ TDatasetSortedByEditor }

procedure TDatasetSortedByEditor.Edit;
var
Dataset: TSqlitePassDataset;
F: TSqlitePassSortedByDialog;
begin
Dataset:=TSqlitePassDataset(GetComponent(0));
if Dataset.Active then
   begin
   F := TSqlitePassSortedByDialog.Create(Nil, Dataset.SortedBy, Dataset.Fields);
   if (F.ShowModal = mrOk)
      then Dataset.SortedBy := F.SQL;
   F.Free;
   end
   else MessageDlg('SortedBy : ' + Msg1015, mtInformation, [mbOk], 0);
end;

function TDatasetSortedByEditor.GetAttributes: TPropertyAttributes;
begin
   Result:= [paDialog];
end;


{ TDatasetFilterEditor }

procedure TDatasetFilterEditor.Edit;
var
Dataset: TSqlitePassDataset;
F: TSqlitePassFilterDialog;
begin
Dataset:=TSqlitePassDataset(GetComponent(0));
if Dataset.Active then
   begin
   F := TSqlitePassFilterDialog.Create(Nil, Dataset);
   F.ShowModal;
   F.Free;
   end
   else MessageDlg('Filter : ' + Msg1015, mtInformation, [mbOk], 0);
end;

function TDatasetFilterEditor.GetAttributes: TPropertyAttributes;
begin
   Result:= [paDialog];
end;



{ TDatasetIndexesEditor }

procedure TDatasetIndexesEditor.Edit;
var
Dataset: TSqlitePassDataset;
F: TSqlitePassIndexDefsDialog;
begin
Dataset:=TSqlitePassDataset(GetComponent(0));
F := TSqlitePassIndexDefsDialog.Create(Nil, Dataset);
F.ShowModal;
F.Free;
end;

function TDatasetIndexesEditor.GetAttributes: TPropertyAttributes;
begin
   Result:= [paDialog];
end;


{ TDatasetMasterDetailFieldsEditor }

procedure TDatasetMasterDetailFieldsEditor.Edit;
var
Dataset: TSqlitePassDataset;
F: TSqlitePassMasterDetailDialog;
begin
Dataset:=TSqlitePassDataset(GetComponent(0));
if Not Assigned(Dataset.MasterSource) then
   begin
   MessageDlg(Msg1000, mtInformation, [mbOk], 0);
   Exit;
   end;

F := TSqlitePassMasterDetailDialog.Create(Nil, Dataset);
F.ShowModal;
F.Free;
end;

function TDatasetMasterDetailFieldsEditor.GetAttributes: TPropertyAttributes;
begin
  Result:= [paDialog];
end;

{ TDatasetFieldDefsEditor }

procedure TDatasetFieldDefsEditor.Edit;
var
Dataset: TSqlitePassDataset;
F: TSqlitePassFieldDefsDialog;
begin
Dataset:=TSqlitePassDataset(GetComponent(0));
F := TSqlitePassFieldDefsDialog.Create(Nil, Dataset);
F.ShowModal;
F.Free;
end;

function TDatasetFieldDefsEditor.GetAttributes: TPropertyAttributes;
begin
  Result:= [paDialog];
end;



{ ----------  TSQLiteDatabase Editors ---------- }

{ TDatabaseFileNameEditor }

procedure TDatabaseFileNameEditor.Edit;
var
D: TOpenDialog;
begin
D := TOpenDialog.Create(Nil);
D.InitialDir := LastDatabaseDir;
D.Filter := 'Kexi Database files (*.kexi)|*.kexi|Sqlite Administrator Database files (*.s3db)|*.s3db|'
          + 'Sqlite Expert Database files (*.db3)|*.db3|All files (*.*)|*.*';
if D.Execute then
   begin
   SetStrValue(D.FileName);
   LastDatabaseDir := ExtractFilePath(D.FileName);
   end;
D.Free;
end;

function TDatabaseFileNameEditor.GetAttributes: TPropertyAttributes;
begin
   Result:= [paDialog];
end;


{ TDatabaseSQLiteLibraryEditor }

procedure TDatabaseSQLiteLibraryEditor.Edit;
var
D: TOpenDialog;
begin
D := TOpenDialog.Create(Nil);
{$IFDEF Windows}
D.Filter := 'SQLite windows library files (*.dll)|*.dll|All files (*.*)|*.*';
{$ELSE}
D.Filter := 'SQLite library files (*.so)|*.so|All files (*.*)|*.*';
{$ENDIF}

if D.Execute
   then SetStrValue(D.FileName);

D.Free;
end;

function TDatabaseSQLiteLibraryEditor.GetAttributes: TPropertyAttributes;
begin
   Result:= [paDialog];
end;


{ TDatabaseDataTypesEditor }

procedure TDatabaseDataTypesEditor.Edit;
var
Database: TSqlitePassDatabase;
F: TSqlitePassDataTypesDlg;
begin
Database:=TSqlitePassDatabase(TSqlitePassDatabaseDataTypeOptions(GetComponent(0)).GetOwner);
F := TSqlitePassDataTypesDlg.Create(Nil, Database.DatatypeOptions, Database.DatabaseType);
F.ShowModal;
F.Free;
end;

function TDatabaseDataTypesEditor.GetAttributes: TPropertyAttributes;
begin
  Result:= [paDialog];
end;

{ TDatabaseCustomFieldDefsEditor }

procedure TDatabaseCustomFieldDefsEditor.Edit;
var
Database: TSqlitePassDatabase;
F: TSqlitePassCustomFieldDefsDlg;
begin
Database:=TSqlitePassDatabase(TSqlitePassDatabaseDataTypeOptions(GetComponent(0)).GetOwner);
F := TSqlitePassCustomFieldDefsDlg.Create(Nil, Database.DatatypeOptions);
F.ShowModal;
F.Free;
end;

function TDatabaseCustomFieldDefsEditor.GetAttributes: TPropertyAttributes;
begin
  Result:= [paDialog];
end;


{ TDatabaseDatatypesComponentEditor }

procedure TDatabaseDatatypesComponentEditor.ExecuteVerb(Index: Integer);
var
Database: TSqlitePassDatabase;
begin
Database:=TSqlitePassDatabase(Component);
  Case Index of
   0: Database.DatatypeOptions.LoadFromDatabase(Database.DatatypeOptions.LoadOptions);
   1: Database.DatatypeOptions.SaveToDatabase(Database.DatatypeOptions.SaveOptions);
   3: Database.DatatypeOptions.SetDefaultValues(Database.DatabaseType, True);
   end;
end;

function TDatabaseDatatypesComponentEditor.GetVerb(Index: Integer): string;
begin
  Case Index of
   0: Result := '&Load DataTypesOptions from database';
   1: Result := '&Save DataTypesOptions to database';
   2: Result := '-';
   3: Result := 'Load &Default translation rules';
   end;
end;

function TDatabaseDatatypesComponentEditor.GetVerbCount: Integer;
begin
  Result := 4;
end;

initialization
{$IFDEF FPC}
 {$i SqlitePassDbo.lrs}
{$ENDIF}
end.


