{ This library is free software; you can redistribute it and/or modify it
  under the terms of the GNU Library General Public License as published by
  the Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public License
  for more details.

  You should have received a copy of the GNU Library General Public License
  along with this library; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

  ---------------------------------------------------------------------------

    Author : Luc DAVID Email: luckylazarus@free.fr
    Copyright (C) 2007
    Last update : 28/02/2007

  --------------------------------------------------------------------------- }

unit SqlitePassFiltersDialog;
{$IFDEF FPC}
 {$mode DELPHI}
{$ENDIF}

interface

uses
 {$IFDEF FPC}
  LResources,
 {$ELSE}
  Windows, Messages,
 {$ENDIF}
  SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Buttons, db, Math,
  SqlitePassDbo;

type
  TSqlitePassFilterDialog = class(TForm)
    PanelIndexesToolBar: TPanel;
    PanelIndexMain: TPanel;
    Splitter1: TSplitter;
    PanelAvailableIndexes: TPanel;
    ListBoxFields: TListBox;
    PanelSQL: TPanel;
    MemoSQLFilterStmt: TMemo;
    PanelIndexCreateStmt: TPanel;
    Label21: TLabel;
    Image4: TImage;
    PanelIndexApplyChanges: TPanel;
    SbCancel: TSpeedButton;
    PanelCaptionAvailableIFields: TPanel;
    SbOk: TSpeedButton;
    Image5: TImage;
    Panel1: TPanel;
    CheckBoxUseQuotes: TCheckBox;
    SbApplyFilter: TSpeedButton;
    procedure SbOkClick(Sender: TObject);
    procedure SbApplyFilterClick(Sender: TObject);
    procedure SbCancelClick(Sender: TObject);
    procedure ListBoxFieldsDblClick(Sender: TObject);
  private
    MyDb: TSqlitePassDatabase;
    MyDataset: TSqlitePassDataset;
    PreviousFilter: String;
    procedure RefreshFieldsList;
  public
    constructor Create(AOwner: TComponent; Dataset: TSqlitePassDataset); reintroduce;
  end;

var
  SqlitePassFilterDialog: TSqlitePassFilterDialog;

implementation

uses RenameItem, CreateNewIndex;

{$IFNDEF FPC}
  {$R *.DFM}
{$ENDIF}


constructor TSqlitePassFilterDialog.Create(AOwner: TComponent; Dataset: TSqlitePassDataset);
begin
inherited Create(AOwner);

ListBoxFields.Clear;
if Dataset = nil
   then abort
   else begin
        MyDataset := Dataset;
        Caption := 'Filter Editor for ' + MyDataset.DatasetName;
        MyDb := MyDataset.Database;
        PreviousFilter := MyDataset.Filter;
        MemoSQLFilterStmt.Text := MyDataset.Filter;
        RefreshFieldsList;
        end;
end;


{ Retrieve data on indexes and fill the listbox with it }
procedure TSqlitePassFilterDialog.RefreshFieldsList;
var
i: integer;
begin
{ no Fields defined ? }
//if MyDataset.Fields.count = 0 then
//   begin
   { We fill the listbox with indexes names }
   For i := 0 to Pred(MyDataset.Fields.count) do
   ListBoxFields.items.Add(MyDataset.Fields[i].FieldName);
//   end
end;

procedure TSqlitePassFilterDialog.SbOkClick(Sender: TObject);
begin
MyDataset.Filter := MemoSQLFilterStmt.Lines.Text;
ModalResult := mrOk;
end;

procedure TSqlitePassFilterDialog.SbApplyFilterClick(
  Sender: TObject);
begin
MyDataset.Filter := MemoSQLFilterStmt.Lines.Text;
MyDataset.ApplyFilter;
end;

procedure TSqlitePassFilterDialog.SbCancelClick(
  Sender: TObject);
begin
MyDataset.Filter := PreviousFilter;
ModalResult := mrCancel;
end;


procedure TSqlitePassFilterDialog.ListBoxFieldsDblClick(Sender: TObject);
var
SelectedField: String;
begin
SelectedField := ListBoxFields.Items[ListBoxFields.ItemIndex];
if CheckBoxUseQuotes.Checked
   then MyDataset.SQLSelectStmt.QuoteString(SelectedField);

MemoSQLFilterStmt.SetFocus;
MemoSQLFilterStmt.SelText := SelectedField;
end;


initialization
 {$IFDEF FPC}
  {$I SqlitePassFiltersDialog.lrs}
 {$ENDIF}
end.