{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: SoLFormFindData.PAS, released on 17-feb-2005.

The Initial Developer of the Original Code is Luc DAVID,
Portions created by Luc DAVID are Copyright (C) 2005 Luc DAVID.
All Rights Reserved.

Contributor(s):


You may retrieve the latest version of this file at the Project SoL Library home page,
located at http://sourceonline.free.fr

Known Issues:
-----------------------------------------------------------------------------}
unit SoLFormFindData;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons, ExtCtrls, DB, DBCtrls, DBGrids, DBTables,
  ActnList, StdActns, ComCtrls, SoLDBActions, Math, SoLRessources;


type
  TFSoLFindData = class(TForm)
    LabelTextToFind: TLabel;
    SbSearch: TSpeedButton;
    SbClose: TSpeedButton;
    SbNext: TSpeedButton;
    TextToFind: TComboBox;
    GroupBoxOptions: TGroupBox;
    CbActiveFieldOnly: TCheckBox;
    CbloCaseSensitive: TCheckBox;
    CbStartBOF: TCheckBox;
    SbPrevious: TSpeedButton;
    LbResearchType: TListBox;
    SpeedButton2: TSpeedButton;
    LabelTextToFindOptions: TLabel;
    procedure SbSearchClick(Sender: TObject);
    procedure SbCloseClick(Sender: TObject);
    procedure FormShortCut(var Msg: TWMKey; var Handled: Boolean);
    procedure FormActivate(Sender: TObject);
    procedure TextToFindChange(Sender: TObject);
    procedure SbNextClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure SbPreviousClick(Sender: TObject);
    procedure LbResearchTypeClick(Sender: TObject);
  private
    Function SearchDataset(const sValue: String; StartBOF: Boolean; ResearchType: Word): Boolean;
  public
    ActiveControl: TSoLActiveControlWatcher;
//    Constructor Create(AOwner: TComponent; DBToolBar: TSoLDBToolBar; ActiveControl: TSoLActiveControlWatcher);
    Constructor Create(AOwner: TComponent; ActiveControl: TSoLActiveControlWatcher); reintroduce;
  end;

var
  FSoLFindData: TFSoLFindData;

implementation

{$R *.DFM}

constructor TFSoLFindData.Create(AOwner: TComponent; ActiveControl: TSoLActiveControlWatcher);
begin
Inherited Create(AOwner);
Self.ActiveControl := ActiveControl;
end;

procedure TFSoLFindData.FormCreate(Sender: TObject);
begin
LabelTextToFind.Caption := SoLRes_SoLFormFindDataFindCaption;
GroupBoxOptions.Caption := SoLRes_SoLFormFindGroupBoxOptions;
LabelTextToFindOptions.Caption := SoLRes_SoLFormFindDataLabelTextToFindOptions;
CbActiveFieldOnly.Caption := SoLRes_SoLFormFindDataCbActiveFieldOnlyCaption;
CbloCaseSensitive.Caption := SoLRes_SoLFormFindDataCbloCaseSensitiveCaption;
CbStartBOF.Caption := SoLRes_SoLFormFindDataCbStartBOFCaption;
SbSearch.Caption := SoLRes_SoLFormFindDataBtnSearchCaption;
SbPrevious.Caption := SoLRes_SoLFormFindDataBtnPreviousCaption;
SbNext.Caption := SoLRes_SoLFormFindDataBtnNextCaption;
SbClose.Caption := SoLRes_SoLFormFindDataBtnCloseCaption;

With LbResearchType do
begin
Clear;
Items.Add(SoLRes_SoLFormFindDataSearchType0);
Items.Add(SoLRes_SoLFormFindDataSearchType1);
Items.Add(SoLRes_SoLFormFindDataSearchType2);
Items.Add(SoLRes_SoLFormFindDataSearchType3);
ItemIndex := 0;
end;
end;

procedure TFSoLFindData.FormActivate(Sender: TObject);
begin
If Assigned(ActiveControl) and ActiveControl.Connected then
   begin
   Height := 290;
   Caption := SoLRes_SoLFormFindDataSelectedField + ActiveControl.DataField.DisplayName;
   TextToFindChange(Sender);
   TextToFind.SetFocus;
   TextToFind.SelectAll;
   end
   else begin
   Height := 0;
   Caption := SoLRes_SoLFormFindDataNoFieldSelected;
   end
end;

{*******************************************************
*** Recherche sur un champs non index d'un TDataset ***
********************************************************}
procedure TFSoLFindData.SbSearchClick(Sender: TObject);
var
RecordFound: Boolean;
Cursor: TCursor;
begin
   Cursor := Screen.Cursor;
   Screen.Cursor := crHourGlass;
   TextToFind.SelectAll;
   RecordFound := SearchDataset(TextToFind.Text,(CbStartBOF.Checked and (SbSearch.enabled = True)), LbResearchType.ItemIndex);
   Screen.Cursor := Cursor;
   If not RecordFound then
      begin
      if not CbStartBOF.Checked then  // on est parti de l'enre en cours
        begin
        If MessageDlg(SoLRes_SoLFormFindDataEndOfSearch, mtConfirmation, [mbYes, mbNo], 0)
        = mrYes then begin
                CbStartBOF.Checked := True;
                SbSearchClick(Self);
                end;
      end;
   end;
ActiveControl.Dataset.EnableControls;
end;

procedure TFSoLFindData.SbCloseClick(Sender: TObject);
begin
ActiveControl.Locked := False;
Close;
end;


procedure TFSoLFindData.TextToFindChange(Sender: TObject);
begin
SbSearch.Enabled := (Length(TextToFind.Text) > 0) and (ActiveControl.DataField <> nil);
SbPrevious.Enabled := False;
SbNext.Enabled := False;
end;

{*****************************************************
*** Recherche sur un champs non index d'une query ***
*****************************************************}
Function TFSoLFindData.SearchDataset(const sValue: String; StartBOF: Boolean; ResearchType: Word): Boolean;
var
RecordPos: TBookMark;
StringPos: Integer;
Label Fin;

begin
Result := False;
If (not ActiveControl.Connected) or (ActiveControl.Dataset.FieldDefs.IndexOf(ActiveControl.DataField.FieldName) < 0)
    then Exit;

RecordPos := ActiveControl.DataSet.GetBookMark;

Try
ActiveControl.Dataset.DisableControls;
If StartBOF and (SbSearch.Enabled)
   then ActiveControl.Dataset.First
   else ActiveControl.Dataset.Next;

While not ActiveControl.DataSet.EOF do
   Begin
      Case ResearchType of
      0 : Result := (Pos(SValue,ActiveControl.DataField.AsString) = 1)
          or (Pos(UpperCase(SValue),ActiveControl.DataField.AsString) = 1);
      1 : Result := (Pos(SValue,ActiveControl.DataField.AsString) <> 0)
          or (Pos(UpperCase(SValue),ActiveControl.DataField.AsString) <> 0);
      2 : begin
          StringPos := Max(1, Length(ActiveControl.DataField.AsString) - Length(SValue) + 1);
          Result := (Pos(SValue,ActiveControl.DataField.AsString) = StringPos)
          or (Pos(UpperCase(SValue),ActiveControl.DataField.AsString) = StringPos);
          end;
      3 : Result := CompareText(SValue,ActiveControl.DataField.AsString) = 0;
      end;
   If Result then
      begin
       SbSearch.Enabled := False;
       SbPrevious.Enabled := True;
       SbNext.Enabled := True;
       Goto Fin;
      end
      else ActiveControl.DataSet.Next;
   end;

{Aucun Resultat}
If (Not Result) then
      begin
      MessageDlg (SoLRes_SoLFormFindDataAllRecordSearched + SValue + '"...', mtInformation, [mbOk], 0);
      ActiveControl.DataSet.GotoBookMark(RecordPos);
    end;

Fin:
Finally
ActiveControl.DataSet.FreeBookMark(RecordPos);
ActiveControl.DataSet.EnableControls;
end;
end;

procedure TFSoLFindData.FormShortCut(var Msg: TWMKey; var Handled: Boolean);
begin
Case Msg.CharCode of
   VK_ESCAPE: begin
              Handled := True;
              Close;
              end;
   VK_DOWN, VK_RETURN:
              begin
              Handled := True;
              if (sbNext.Enabled or sbSearch.Enabled) then sbSearchClick(SbNext)
              end;
   VK_UP:     begin
              Handled := True;
              if (sbPrevious.Enabled or sbSearch.Enabled)  then sbSearchClick(SbPrevious)
              end;
   end;
Handled := False;
end;

procedure TFSoLFindData.SbPreviousClick(Sender: TObject);
begin
sbSearchClick(Sender);
end;


procedure TFSoLFindData.SbNextClick(Sender: TObject);
begin
sbSearchClick(Sender);
end;

(*
      begin
      (*TForm(ActiveControl.Control.Owner).SetFocus;
      if ActiveControl.Control.Visible and ActiveControl.Control.Enabled
         then ActiveControl.Control.SetFocus;
      if ActiveControl.Control is TDBEdit then
         begin
         TDBEdit(ActiveControl.Control).SelStart := Pos(UpperCase(TextToFind.Text),UpperCase(ActiveControl.DataField.asString))-1;;
         TDBEdit(ActiveControl.Control).SelLength := Length(TextToFind.Text);
         end;
      end
   else

//True if prop property exists for comp component
function HasProperty(comp:TComponent;prop:string):boolean;
begin
	result:=(getPropInfo(comp.classInfo,prop)<>nil) and (comp.name<>'');
end;

procedure TKADaoSearch.OnSearchClick(Sender: TObject);
Var
 KeyFields : string;
 KeyValues : Variant;
 Options   : TLocateOptions;
 DaoTable  : TKADaoTable;
begin
 KeyFields := F_SCombo.Items.Strings[F_SCombo.ItemIndex];
 KeyValues := F_SEdit.Text;
 Options   := [loCaseInsensitive, loPartialKey];
 DaoTable  := (F_DataLink.Dataset as TKADaoTable);
 if NOT Seek Then
    Begin
      if Not DaoTable.Find_Next (KeyFields, KeyValues, Options) Then
             DaoTable.Find_First(KeyFields, KeyValues, Options);
      VarClear(KeyValues);
    End
 Else
    Begin
      DaoTable.Locate(KeyFields, KeyValues, Options);
      VarClear(KeyValues);
    End;
end;


(*  Field: TField; var Enabled: Boolean);
begin
  Enabled := (TRxDBGrid(Sender).DataSource.DataSet is TTable) and
    (Field <> nil) and not (Field is TBlobField) and
    (TTable(TRxDBGrid(Sender).DataSource.DataSet).IndexDefs.Count > 0);
end;

procedure TMDIChild.rxDBGrid2TitleBtnClick(Sender: TObject; ACol: Longint;
  Field: TField);
begin
  if TRxDBGrid(Sender).DataSource.DataSet is TTable then
  try
    TTable(TRxDBGrid(Sender).DataSource.DataSet).IndexFieldNames :=
      Field.FieldName;
  except
    TTable(TRxDBGrid(Sender).DataSource.DataSet).IndexFieldNames := '';
  end;
end;

procedure TMDIChild.rxDBGrid2GetBtnParams(Sender: TObject; Field: TField;
  AFont: TFont; var Background: TColor; IsDown: Boolean);
begin
  if (TRxDBGrid(Sender).DataSource.DataSet is TTable) and (Field <> nil) and
    (Field.IsIndexField) then
  begin
    Background := $00D3D3D3;
  end;
end;

procedure TFSearchDb.FillDbLists;
var
i: Word;
c: TComponent;
s: TStringList;
begin
s := TStringList.Create;
s.Sorted := True;
s.Duplicates := dupIgnore;
For i:= 0 to Screen.ActiveForm.ComponentCount-1 do
  begin
  c:= Screen.ActiveForm.Components[i];
  if (c is TDBEdit)
     then if (c as TDBEdit).DataSource.Dataset.active = true
          then s.Add((c as TDBEdit).DataSource.name);

  if (c is TDBGrid)
     then if (c as TDBGrid).DataSource.Dataset.active = true
          then s.Add((c as TDBGrid).DataSource.name);

  end;
DatasetsList.Items.Assign(s);
s.Free
end;*)


procedure TFSoLFindData.LbResearchTypeClick(Sender: TObject);
begin
TextToFindChange(Sender);
end;

end.
