{ This library is free software; you can redistribute it and/or modify it
  under the terms of the GNU Library General Public License as published by
  the Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public License
  for more details.

  You should have received a copy of the GNU Library General Public License
  along with this library; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

  ---------------------------------------------------------------------------

    Author : Luc DAVID Email: luckylazarus@free.fr
    Copyright (C) 2006 - 2008

    Major changes are indicated in the \Documentation\Changes.pdf file
    Last update 25.02.2008

  --------------------------------------------------------------------------- }


{ TSqlitePassMasterDataLink }

{ Synchronises Recordset fields order with Dataset fields }

procedure TSqlitePassMasterDataLink.LayoutChanged;
begin
  TSqlitePassDataset(Dataset).UpdateFieldsNo;
  Inherited;
end;


{ TSqlitePassDatasetVersionInfo }

constructor TSqlitePassDatasetVersionInfo.Create;
begin
inherited Create;
FComponent := SqlitePassDatasetVersion;
FPackage := SqlitePassPackageVersion;
end;

procedure TSqlitePassDatasetVersionInfo.SetFComponent(
  const AValue: String);
begin
{ Read only - Used to display infos in object inspector }
end;

procedure TSqlitePassDatasetVersionInfo.SetFPackage(const AValue: String);
begin
{ Read only - Used to display infos in object inspector }
end;


{ --- TSqlitePassDataset --- }

constructor TSqlitePassDataset.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if Assigned(AOwner) and (AOwner is TSqlitePassDatabase)
     then FDatabase := TSqlitePassDatabase(AOwner)
     else FDatabase := nil;
  FDatasetName := '';
  FDatasetType := dtUnknown;
  FInInternalOpen := False;
  FReadOnly := False;
  FRecordset := TSqlitePassRecordset.Create(Self);
  { Locate }
  FLocateRecordset := TSqlitePassRecordset.Create(Self);
  FLocateSQL := TSqlitePassSelectStmt.Create(Self);
  FLocateTable := TList.Create;
  { }
  FProcessMessages := False;
  FSQLChanged := False;
  FSQL := TStringList.Create;
  FSQL.Clear;
  FSQLSelectStmt := TSqlitePassSelectStmt.Create(Self);
  SetFSortedBy('');
  FExportMethod := VisibleFields;
  FFieldDefsInfo := Msg1100;
  FIndexDefs := TSqlitePassDatasetIndexDefs.Create(Self);
  FVersionInfo := TSqlitePassDatasetVersionInfo.Create;
  FDatabaseAutoActivate := True;
  FParams := TParams.Create(Self);
  { Filtering }
  FOnFilterRecord := nil;
  FDetailFields := TList.Create;
  FMasterLink := TSqlitePassMasterDataLink.Create(Self);
  FMasterLink.OnMasterChange := MasterChanged;
  FMasterLink.OnMasterDisable := MasterDisabled;
  FFilterRecordLowerLimit := 0;
  FFilterRecordUpperLimit := 0;
  FMasterAutoActivate   := True;
  { BookmarkSize is used by TDataset }
  BookmarkSize := SizeOf(Integer);
  { Ensure correct info to FSQLStmt with FSQL.assign or gettext/settext }
  FSQL.OnChange := UpdateFSQL;
  FOnExportProgress := nil;
  FOnImportProgress := nil;
End;

destructor TSqlitePassDataset.Destroy;
Begin
  Close;

  { Force Database.Datasets to remove this dataset from the datasets list }
  Database := nil;

  FVersionInfo.Free;
  FIndexDefs.Free;
  FParams.Free;
  OnFilterRecord := nil;
  FMasterLink.OnMasterChange := nil;
  FMasterLink.OnMasterDisable := nil;
  FMasterLink.Free;
  FDetailFields.Free;
  FSQLSelectStmt.Free;
  FSQL.OnChange := nil;
  FSQL.Free;
  FLocateSQL.Free;
  FLocateRecordset.Free;
  FLocateTable.Free;
  FRecordset.Free;
//  InternalClearFieldDefs;
   // TODO Check FreeNotification / RemoveFreeNotification
  if Assigned(FDatabase) then FDatabase.FreeNotification(Self);
     {$IFDEF FPC}
    //FDatabase.RemoveFreeNotification(Self);
   {$ENDIF}
  inherited Destroy;
End;

Function TSqlitePassDataset.GetFDatabase:TSqlitePassDatabase;
Begin
 Result:=FDatabase;
End;

Procedure TSqlitePassDataset.SetFDatabase(Value:TSqlitePassDatabase);
Begin
 if Active then DatabaseError(Msg2011);

 { Previous FDatabase }
 if Assigned(FDatabase) then
    begin
    { We remove the previous dataset from the Database datasets list }
    FDatabase.Datasets.Remove(Self);
    // TODO : Check FSQLSelectStmt.FDatabase assignement
    FSQLSelectStmt.FDatabase:= nil;
    end;

 FDatabase := Value;

 { New Database }
 if Assigned(FDatabase) then
    begin
    { We add this dataset to the Database datasets list }
    FDatabase.Datasets.Add(Self);
    FSQLSelectStmt.FDatabase:= FDatabase;
    end;
End;


Procedure TSqlitePassDataset.SetFDatasetName(const Value: String);
var
IsActive: Boolean;
Begin
if Value = FDatasetName
   then Exit;

(*if (csDesigning in ComponentState) and (FDatasetName <> '') and (Fields.Count > 0)
   then if MessageDlg(Msg2028, mtWarning, [mbYes, mbNo], 0) = idYes
           then DestroyFields
           else Exit;*)
if (csDesigning in ComponentState) and (FDatasetName <> '') and (Fields.Count > 0)
   then if MessageDlg(Msg2028, mtWarning, [mbYes, mbNo], 0) = idNo
           then Exit;

FDatasetFullName := Value;
FDatasetName := Copy(Value, Pos('.',Value)+1,Length(Value));

{ Resets properties which are specicific to a dataset
  and avoid to call Dataset.Close / Open twice if not csLoading }
if not (csLoading in ComponentState) then
   begin
   IsActive := Active;
   if IsActive
       then Close;
   DestroyFields;
   Filtered := False;
   Filter := '';
   FilterRecordLowerLimit := 0;
   FilterRecordUpperLimit := 0;
   Sorted := False;
   SortedBy := '';
   MasterSource := nil;
   SetFDatasetType;
   Active := IsActive;
   end
   else SetFDatasetType;
End;

procedure TSqlitePassDataset.SetFDatasetType;
var
IsActive: Boolean;
i: integer;
SelectedFields, FieldIdentifier, TableIdentifier: string;
begin
FDatasetType := dtUnknown;
if (Database = nil) or ((Not Database.Connected) and (Database.FState = dbsInactive)) then exit;

Try
{ disable notification }
FSQL.OnChange := nil;
IsActive := Active;
if IsActive
   then Close;

if FDatasetName = ''
   then FDatasetType := dtSQLDirect
   else if FDatabase.TableDefs.FindTable(FDatasetFullName) <> nil
           then FDatasetType := dtTable
           else if FDatabase.QueryDefs.FindQuery(FDatasetFullName) <> nil
                   then FDatasetType := dtQuery
                   else if FDatabase.Views.FindView(FDatasetFullName) <> nil
                        then FDatasetType := dtView;
Case FDatasetType of
     dtTable:   begin
                UpdateIndexDefs;
                TableIdentifier := FDatasetFullName;
                FSQLSelectStmt.QuoteString(TableIdentifier);
                { We check if the designtime designer is active.
                  If so we select all the table fields to display them in the field designer
                  Else we select only the wanted fields to save memory }
                If Assigned(Fields) and (Fields.Count > 0) and not (csDesigning in ComponentState)
                   then begin
                        For i := 0 to Pred(Fields.Count)
                            do if Fields[i].FieldKind = fkData
                                  then begin
                                       FieldIdentifier := Fields[i].FieldName;
                                       FSQLSelectStmt.QuoteString(FieldIdentifier);
                                       SelectedFields  := SelectedFields + FieldIdentifier + ', ';
                                       end;
                        System.Delete(SelectedFields,Length(SelectedFields)-1,2);
                        end
                   else SelectedFields := '*';

                   FSQL.Text := 'SELECT ' +  SelectedFields + ' FROM ' + TableIdentifier + ';'
                end;
     dtQuery:   begin
                FSQL.Text := Fdatabase.QueryDefs.QueryByName(FDatasetFullName).Sql;
                UpdateParamsList;
                end;
     dtView:    FSQL.Text := Fdatabase.Views.ViewByName(FDatasetName).SqlSelectStmt;
     else       begin { Dataset name doesn't exist in database ?}
                FDatasetFullName := '';
                FDatasetName := '';
                end;
     end;
Finally
{ Enable notification }
FSQL.OnChange := UpdateFSQL;

Active := IsActive;

{ if the dataset is closed, force fielddefs update }
if Not Active
   then InternalInitFieldDefs;
end;
end;

procedure TSqlitePassDataset.SetFSQL(const Value: TStringList);
begin
{ This is necessary to avoid error at designtime... }
if Value.Text <> FSQL.Text
   then FSQL.Assign(Value);
end;

{ This procedure is called whenever a change is made to FSQL
  via TStringList(FSQL).OnChange := UpdateFSQL,
  to ensure correct assignement with FSQL.assign or gettext/settext }
Procedure TSqlitePassDataset.UpdateFSQL(Sender: TObject);
Begin
if (csLoading in ComponentState) then exit;
{ We consider, once the SQL statement is changed by end user,
  that the Dataset doesn't match a known database entry (table, query or view)
  Setting the DatasetName to '' will call SetFDatasetName and SetFDatasetType }
DatasetName := '';
FSQLChanged := True;
End;

{ This procedure should be called whenever a change is made to FSQL
  to ensure correct info to FSQLStmt }
Procedure TSqlitePassDataset.UpdateFSQLStmt;
var
i: Integer;
Stmt, MasterDetailStmt: string;
Begin
{ we need to store the string to a local variable }
Stmt := FSQL.Text;

FSQLSelectStmt.Prepare(Stmt);

{ Master/Detail Filtering }
MasterDetailStmt := '';
if (MasterFields <> '') and (FInInternalOpen or Active) and Assigned(FDetailFields)
   and Assigned(FMasterLink.Datasource) and (FMasterLink.Active) then
   begin
   For i := 0 to Pred(FDetailFields.Count)
       do MasterDetailStmt := TField(FDetailFields[i]).FieldName + '="' + TField(FMasterLink.Fields[i]).AsString + '"';

   FSQLSelectStmt.ReplaceSection(kwWhere, kwWhere + ' ' + MasterDetailStmt);
   end;

{ Filtering }
if Filtered then
   begin
   if MasterDetailStmt <> ''
      then FSQLSelectStmt.ReplaceSection(kwWhere, kwWhere + ' (' + MasterDetailStmt + ') AND ' + Filter)
      else if Filter <> ''
              then FSQLSelectStmt.ReplaceSection(kwWhere, kwWhere + ' ' + Filter);
   end;

{ Sorting }
if Sorted and (Length(FSortedBy) > 0)
  then FSQLSelectStmt.ReplaceSection(KwOrderBy, kwOrderBy + ' ' + FSortedBy);

FSQLChanged := False;
{$IFDEF DEBUG}
   FDatabase.Debugger.Display('SQL Statement : ' + FSQLSelectStmt.SQL.Text);
{$ENDIF}
end;

Procedure TSqlitePassDataset.UpdateFieldsNo;
begin
if Active
   then FRecordset.SetFieldsNo(Fields);
Inherited;
end;

Function TSqlitePassDataset.GetFReadOnly: Boolean;
Begin
If(Assigned(FDatabase) and FDatabase.ReadOnly)
   then FReadOnly := True;

If Active and (not (DatasetType in [dtTable, dtQuery, dtSQLDirect]))
   then FReadOnly := True;

Result  := FReadOnly;
End;

Procedure TSqlitePassDataset.SetFReadOnly(const Value:Boolean);
Begin
  FReadOnly := Value;
End;

Procedure TSqlitePassDataset.Loaded;
begin
try
  {DatasetType cannot be set properly while TSqlitePassDataset
   is loading from DFM or LFM. So we do it now }
  SetFDatasetType;
  inherited Loaded;
except
  Application.HandleException(Self)
end;
end;

Procedure TSqlitePassDataset.Notification(AComponent: TComponent; Operation: TOperation); //Ok
Begin
 If (Operation = opRemove) and (AComponent = FDatabase) then FDatabase := Nil;
 Inherited Notification(AComponent, Operation);
End;


function TSqlitePassDataset.CheckCanOpen(ShowError: Boolean = True): Boolean;
var
Msg: String;

begin
Result := Active or FInInternalOpen;

if Not Result then
   begin
     Try
     if not Assigned(FDatabase) then
        begin
        Msg := Msg2018;
        Abort;
        end;

     if FDatabaseAutoActivate and (FDatabase.FState = dbsInactive)
        then FDatabase.Connected:=True;

     if Not (FDatabase.Connected) and (FDatabase.FState = dbsInactive) then
        begin
        Msg := Msg2020;
        Abort;
        end;

     if (FDatasetName='') And (FSQL.Count=0)
        then Msg := Msg2019;

     finally
     Result := (Msg = '');
     If Not Result And ShowError
        then DatabaseError(Msg);
     end;
   end;
end;

{procedure InternalOpen; virtual; abstract;

 TDataset.Open -> DoInternalOpen -> InternalOpen
------------------------------------------
Open the dataset. You must call InternalInitFieldDefs;
if DefaultFields is True, then you must call CreateFields,
which will create the necessary TFields from the fielddefs.}
Procedure TSqlitePassDataset.InternalOpen;
Begin

if Not CheckCanOpen then Exit;

try
  FInInternalOpen := True;

  { Init FieldDefs }
  InternalInitFieldDefs;

  { If fields definitions are stored in *.lfm or *.dfm using design time fields editor,
    then they are created at runtime when the form is created, even if the dataset is closed.
    If no persistent fields have been created in the Fields Editor
    then FPC uses the TDataset.CreateFields (It calls FieldDefs.CreateField) to create fields dynamically. }
  if DefaultFields then CreateFields;

  { TDataset.BindFields sets up all sorts of internal
    things like field offsets, as well as creating any calculated
    or lookup fields. It is within BindFields that CalcFieldsSize
    is actually set for later use in InitBufferPointers }
  BindFields(True);

  { Now Fields are defined, we can process MasterFields }
  if Assigned(MasterSource) then
     begin
     if FMasterAutoActivate and Assigned(MasterSource.Dataset)
        then MasterSource.Dataset.Open;
     ProcessMasterDetailFields;
     end;

  { Execute the Query }
  InternalRefresh;
 //FDatabase.Idle;
finally
  FInInternalOpen := False;
end;
end;

{procedure InternalRefresh; virtual; abstract;
-------------------------------------------
Refresh the dataset.}
procedure TSqlitePassDataset.InternalRefresh;
var
  SaveState: TDataSetState;
begin
   if FSQLChanged
      then UpdateFSQLStmt;

   if Filtered and Assigned(OnFilterRecord)
      then SaveState := SetTempState(dsFilter);

   { Query data using SQL sentence }
   If FSQLSelectStmt.IsValid
      then FRecordset.Open(Database.Engine.Connection, FSQLSelectStmt.SQL.Text)
      else DatabaseError('Bad SQL');

   if State = dsFilter
      then RestoreState(SaveState);
end;

procedure TSqlitePassDataset.RefreshActive;
begin
if Active then Refresh;
end;

Procedure TSqlitePassDataset.InternalClearFieldDefs;
begin
{$IFDEF DEBUG}
   FDatabase.Debugger.Display('InternalClearFieldDefs');
{$ENDIF}
{ Clear the internal items }
While FieldDefs.Count > 0
  do FieldDefs[0].Free;
{ Clear the list pointers }
FieldDefs.Clear;
end;

{procedure InternalClose; virtual; abstract;
-------------------------------------------
Closes the dataset. Any resources allocated in InternalOpen should be freed
here.}
Procedure TSqlitePassDataset.InternalClose;
Begin
FDatasetType := dtUnknown;
FSQLSelectStmt.Unprepare;
FRecordset.Close;
BindFields(False);
if DefaultFields
   then DestroyFields;
End;


{procedure InternalGotoBookmark(ABookmark: Pointer); virtual; abstract;
----------------------------------------------------------------------
Set the record position on the position that is associated with the
ABookMark data. The ABookMark data is the data that is acquired through
the GetBookMarkData call, and should be kept for each record.}
Procedure TSqlitePassDataset.InternalGotoBookmark(ABookmark: Pointer);
begin
 FRecordset.CurrentRecordIndex := PInteger(ABookmark)^;
End;

Function TSqlitePassDataset.BookmarkValid(Bookmark: TBookmark): Boolean;
Begin
Result := (PInteger(Bookmark)^ > -1) and (PInteger(Bookmark)^ < RecordCount);
End;

{ TDataset: function GetBookmarkFlag(Buffer: PChar): TBookmarkFlag; virtual; abstract;
  --------------------------------------------------------------------------
  Returns the BookmarkFlag (bfCurrent, bfBOF, bfEOF, bfInserted)
  associated with a recordBuffer.
  -------------------------------------------------------------------------- }
Function TSqlitePassDataset.GetBookmarkFlag(Buffer: PChar): TBookmarkFlag;
begin
   Result := TSqlitePassMemRecord(PInteger(Buffer)^).BookMarkFlag;
{$IFDEF DEBUG}
   FDatabase.Debugger.Display('GetBookmarkFlag (bfCurrent, bfBOF, bfEOF, bfInserted) : Record = ' + IntToStr(TSqlitePassMemRecord(PInteger(Buffer)^).RecordIndex)
   + ' BookmakFlag := ' + IntToStr(Ord(TSqlitePassMemRecord(PInteger(Buffer)^).BookMarkFlag)));
{$ENDIF}
End;

{TDataset: procedure SetBookmarkFlag(Buffer: PChar; Value: TBookmarkFlag); virtual; abstract;
-----------------------------------------------------------------------------
Set the BookmarkFlag 'Value' (bfCurrent, bfBOF, bfEOF, bfInserted)
of the record Buffer.
----------------------------------------------------------------------------- }
Procedure TSqlitePassDataset.SetBookmarkFlag(Buffer: PChar; Value: TBookmarkFlag);
Begin
TSqlitePassMemRecord(PInteger(Buffer)^).BookMarkFlag := Value;
{ We store the active record index to enable repositionning if insertion is canceled }
if Value = bfInserted
   then FRecordset.PreviousRecordIndex := FRecordset.CurrentRecordIndex;

{$IFDEF DEBUG}
   FDatabase.Debugger.Display('> SetBookmarkFlag (bfCurrent, bfBOF, bfEOF, bfInserted) : Record = ' + IntToStr(TSqlitePassMemRecord(PInteger(Buffer)^).RecordIndex)
   + ' BookmakFlag := ' + IntToStr(Ord(TSqlitePassMemRecord(PInteger(Buffer)^).BookMarkFlag)));
{$ENDIF}
End;

{procedure GetBookmarkData(Buffer: PChar; Data: Pointer); virtual; abstract;
---------------------------------------------------------------------------
Puts the bookmarkdata for Buffer into the area pointed to by Data.}
Procedure TSqlitePassDataset.GetBookmarkData(Buffer: PChar; Data: Pointer);
Begin
if (Buffer <> Nil) and (Data <> Nil)
   then PInteger(Data)^ := TSqlitePassMemRecord(PInteger(Buffer)^).RecordIndex;

{$IFDEF DEBUG}
FDatabase.Debugger.Display('GetBookmarkData : Record = ' + IntToStr(TSqlitePassMemRecord(PInteger(Buffer)^).RecordIndex)
   + ' BookmakFlag := ' + IntToStr(Ord(TSqlitePassMemRecord(PInteger(Buffer)^).BookMarkFlag)));
{$ENDIF}
End;

{procedure SetBookmarkData(Buffer: PChar; Data: Pointer); virtual; abstract;
---------------------------------------------------------------------------
Move the 'Data' value to the BookmarkData associated with Buffer}
Procedure TSqlitePassDataset.SetBookmarkData(Buffer: PChar; Data: Pointer);
Begin
if (Buffer <> Nil) and (Data <> Nil)
   then TSqlitePassMemRecord(PInteger(Buffer)^).RecordIndex := PInteger(Data)^;
{ We store the active record index to enable repositionning if insertion is canceled }
if TSqlitePassMemRecord(PInteger(Buffer)^).BookmarkFlag = bfInserted
   then FRecordset.PreviousRecordIndex := PInteger(Data)^;

{$IFDEF DEBUG}
FDatabase.Debugger.Display('> SetBookmarkData : Record = ' + IntToStr(TSqlitePassMemRecord(PInteger(Buffer)^).RecordIndex)
   + ' BookmakFlag := ' + IntToStr(Ord(TSqlitePassMemRecord(PInteger(Buffer)^).BookMarkFlag)));
{$ENDIF}
End;

Procedure TSqlitePassDataset.InternalInitIndexDefs;
Begin
{$IFDEF DEBUG}
   FDatabase.Debugger.Display('InternalInitIndexDefs');
{$ENDIF}
 UpdateIndexDefs;
End;

{ This procedure is called whenever the TDataset.fields (adding, deleting, ordering...)
  is changed by user.
  It takes care of updating internal FMemRecordset fields position and
  internal SQL statements so we always retrieve or set the right value
  from the active TSqlitePassMemRecordset   }
procedure TSqlitePassDataset.DataEvent(Event: db.TDataEvent; Info: Integer);
begin
if Event = deFieldListChange then
   begin
   FRecordset.SetFieldsNo(Fields);
   { Prevents unnecessary calls when updating fields list while loading Forms and components }
   if Assigned(FDatabase) and Active
      then FSQLSelectStmt.PrepareSQLStmts;
   end;
Inherited DataEvent(Event, Info);
end;

Procedure TSqlitePassDataset.UpdateIndexDefs;
Begin
 FIndexDefs.RefreshFromCache;
End;

{procedure InternalInitFieldDefs; virtual; abstract;
-----------------------------------------------------------------------
This method is called from InternalOpen, and
initialize FieldDef definitions for all fields in a record.
It adds these definitions to the FFielddefs object.
----------------------------------------------------------------------}
Procedure TSqlitePassDataset.InternalInitFieldDefs;
var
i: Integer;
StmtFieldDef: TSqlitePassSelectStmtFieldDef;
NewFieldDef: TFieldDef;
CheckColumnCount, ColumnCount: Integer;
IsActive: Boolean;

  Function GetFieldNo(FieldName: String): Integer;
  var
  i: Integer;
  s: string;
  begin
  Result := -1;
  For i := 0 to Pred(ColumnCount) do
      begin
      S := SqliteDbv3_column_name(SqlSelectStmt.StmtHandle, i);
      if (FieldName = S)
         then begin
              Result := i+1;
              Break;
              end;
      end;
  end;


begin
{$IFDEF DEBUG}
   FDatabase.Debugger.Display('InternalInitFieldDefs');
{$ENDIF}

if not CheckCanOpen(False)
   then Exit;

IsActive := Active;
   
if FDatasetType = dtUnknown
   then SetFDatasetType;

InternalClearFieldDefs;

{ First, we need to process the SQL Statement }
UpdateFSQLStmt;

ColumnCount := SqliteDbv3_column_count(SqlSelectStmt.StmtHandle);
CheckColumnCount := 0;

For i := 0 to Pred(SqlSelectStmt.FieldDefs.Count) do
    begin
    StmtFieldDef := SqlSelectStmt.FieldDefs[i];

    NewFieldDef := TFieldDef.Create(FieldDefs,GetUniqueFieldName(StmtFieldDef.FieldName),StmtFieldDef.DataType,
                                    StmtFieldDef.Size, False, GetFieldNo(StmtFieldDef.FieldName));
    { Then we fill the following properties }
    With NewFieldDef do
      begin
      Precision := StmtFieldDef.Precision;

      { Hides the PrimaryKeys fields if necessary }
      if StmtFieldDef.PrimaryKey and StmtFieldDef.AutoInc then
         begin
         Attributes := Attributes + [faReadOnly];
         if StmtFieldDef.InternalPrimaryKey
            then Attributes := Attributes + [faHiddenCol];
         end;
         
     { This will also set TFieldDef.Attributes +/- [faRequired] }
      //Required := StmtFieldDef.NotNull and Not (faHiddenCol in Attributes) And Not (faReadOnly in Attributes);

      Inc(CheckColumnCount);
      end; { With }
     end;{ For i }

{ Raises an Exception if Sqlite Engine doesn't return the same ColumCount
  as parsed SQL Statement ...}
If ColumnCount <> CheckColumnCount
   then DatabaseError(Msg2025, Self);

 InternalInitIndexDefs;

if not IsActive
   then close;
end;

{procedure InternalHandleException; virtual; abstract;
-----------------------------------------------------
not needed yet. Just implement an empty call.}
Procedure TSqlitePassDataset.InternalHandleException;
begin
 Application.HandleException(Self);
end;


Function TSqlitePassDataset.GetActiveRecord: TSqlitePassMemRecord;
Begin
Try
case State of
     dsBrowse      : if IsEmpty  // Ok
                        then Result := nil
                        else Result := TSqlitePassMemRecord(PInteger(ActiveBuffer)^);
     dsBlockRead,
     dsNewValue,
     dsCurValue,
     dsOldValue,
     dsInsert,
     dsEdit        : Result := TSqlitePassMemRecord(PInteger(ActiveBuffer)^);

     dsCalcFields  : Result := TSqlitePassMemRecord(PInteger(CalcBuffer)^);
     dsFilter      : Result := FRecordset.CurrentRecord;
     dsInactive    : Result := nil;

     else Result := nil;
     End

Except
Result:=nil;
End;
End;


{function GetFieldData(Field: TField; Buffer: Pointer): Boolean; virtual; abstract;
----------------------------------------------------------------------------------
Puts the data for field 'Field' from the active buffer into Buffer.
This is called whenever a field value is demanded by a TField, so it must be
efficient.
the TDataset class doesnt know anything about how our record buffer is organized.
The next question then becomes that if the TDataset class doesnt know about our
record structure, how it does it pull out field values from this structure?
How it does it put them back in when the user is editing a record?
The answer is that it uses the GetFieldData and SetFieldData methods which a
custom dataset class must override.
These methods are called by the Tdataset.fields when it needs to retrieve
or set a specific field value from the current record buffer.
GetFieldData is prototyped as follows:

function TDataset.GetFieldData(Field: TField; Buffer: Pointer): Boolean;

The Field parameter is the field for which the value needs to be retrieved
We send back the needed value in the buffer.}
Function TSqlitePassDataset.GetFieldData(Field: TField; Buffer: Pointer): Boolean;
var
  ActiveRec: TSqlitePassMemRecord;
  FieldNo: Integer;
begin
Try
Result := False;
ActiveRec := GetActiveRecord;
if ActiveRec = nil then exit;
FieldNo := FRecordset.GetFieldNo(Field.Index);
if FieldNo > -1
   then Result := Database.Translator.GetFieldData(ActiveRec.FieldsValues[FieldNo], Field.DataType, Buffer);

{$IFDEF DEBUG}
//FDatabase.Debugger.Display(PChar(DatasetName+'     GetFieldData : Field ' + Field.DisplayName + ' FieldNo  = ' + IntToStr(FieldNo) + ', Value = ' + ActiveRec.FieldsValues[FieldNo] + ', CurrentRecordIndex = ' + IntToStr(ActiveRec.RecordIndex)));
{$ENDIF}

Except
Result := False;
end;
End;


{procedure SetFieldData(Field: TField; Buffer: Pointer); virtual; abstract;
--------------------------------------------------------------------------
Move the data associated with Field from Buffer to the activebuffer.}
Procedure TSqlitePassDataset.SetFieldData(Field: TField; Buffer: Pointer);
var
  ActiveRec: TSqlitePassMemRecord;
  FieldNo: Integer;
Begin
  ActiveRec := GetActiveRecord;
  If ActiveRec = nil then exit;

  FieldNo := FRecordset.GetFieldNo(Field.Index);
  Database.Translator.SetFieldData(Buffer, Field.DataType, ActiveRec.FieldsValues[FieldNo]);
  {$IFDEF DEBUG}
  FDatabase.Debugger.Display('SetFieldData : Field ' + Field.DisplayName + ' FieldNo = '+IntToStr(FieldNo)+ ', CurrentRecordIndex = ' + IntToStr(ActiveRec.RecordIndex));
  {$ENDIF}

   { The DataEvent is used to set TDataset.SetModified to True.
     Then TDataset.CheckBrowseMode will be able to Post the record if necessary
     when scrolling the dataset }
   if not (State in [dsInternalCalc, dsCalcFields, dsFilter, dsNewValue]) then DataEvent(deFieldChange, Longint(Field));
End;

{procedure InternalFirst; virtual; abstract;
-------------------------------------------
This is called when 'TDataset.First' is called;
After this method, TDataset calls 'Getrecord' asking for the next record.
'Next' will inc(CurrentRecordIndex). Finally 'GetRecord' will retrieve
Record[FMemRecordset.CurrentRecordIndex] i.e. Record[0]
or return grBOF if we are already at BOF}
Procedure TSqlitePassDataset.InternalFirst;
Begin
FRecordset.First;
End;

{procedure InternalLast; virtual; abstract;
------------------------------------------
This is called when 'TDataset.Last' is called;
After this method, TDataset calls 'Getrecord' asking for the prior record.
'Prior' will Dec(CurrentRecordIndex). Finally 'GetRecord' will retrieve
Record[FMemRecordset.CurrentRecordIndex] i.e. Record[Pred(RecordCount)]
or return grEOF if we are already at EOF}
Procedure TSqlitePassDataset.InternalLast;
Begin
FRecordset.Last;
End;

{procedure InternalSetToRecord(Buffer: PChar); virtual; abstract;
----------------------------------------------------------------
Set the current record to the record in Buffer; if bookmark data
is specified in this buffer, that data can be used to determine which
record this should be.}
Procedure TSqlitePassDataset.InternalSetToRecord(Buffer: PChar);
Begin
if Buffer = nil then Exit;
FRecordset.CurrentRecord := TSqlitePassMemRecord(PInteger(Buffer)^);
End;

procedure TSqlitePassDataset.InternalInsert;
begin
{ This is just an empty call - could also be empty}
inherited internalInsert;
end;

Procedure TSqlitePassDataset.InternalEdit;
Begin
inherited internalEdit;
FRecordset.SaveCurrentRecordOldValues;
End;

Procedure TSqlitePassDataset.InternalCancel;
Begin
inherited InternalCancel;
Case State of
     dsEdit:   FRecordset.RestoreCurrentRecordOldValues;
     dsInsert: FRecordset.Cancel;
     end;
End;

{procedure InternalPost; virtual; abstract;
------------------------------------------
Post the data in the active buffer to the underlying dataset. }
Procedure TSqlitePassDataset.InternalPost;
begin
Case State of
     dsEdit   : FSQLSelectStmt.Post(ptUpdate);
     dsInsert : If FSQLSelectStmt.Post(ptInsert)
                   then FRecordset.AddRecord;
                   else DatabaseError(Msg2022);
     end;
End;

Procedure TSqlitePassDataset.Resync(Mode: TResyncMode);
Begin
  Inherited Resync(Mode);
End;


{procedure InternalAddRecord(Buffer: Pointer; Append: Boolean); virtual; abstract;
---------------------------------------------------------------------------------
Adds a record to the dataset. The record's data is in Buffer and Append
indicates whether the record should be appended (True) or Inserted (False).
Note that for SQL based datasets, this has no meaning.}
Procedure TSqlitePassDataset.InternalAddRecord(Buffer: Pointer; Append: Boolean);
Begin
  InternalPost;
End;


{procedure InternalDelete; virtual; abstract;
--------------------------------------------
Deletes the current Record.}
Procedure TSqlitePassDataset.InternalDelete;
Begin
If FSQLSelectStmt.Post(ptDelete)
   then FRecordset.Delete(FRecordset.CurrentRecord.RecordIndex)
   else DatabaseError(Msg2024);
End;


{function IsCursorOpen: Boolean; virtual; abstract;
--------------------------------------------------
This function should return True if data is available, even if the dataset
is not active.}
Function TSqlitePassDataset.IsCursorOpen: Boolean;
Begin
  Result:=FRecordset.Active;
End;

{function AllocRecordBuffer: PChar; virtual; abstract;
-----------------------------------------------------
Must allocate enough memory to store a complete record in the dataset.
Optionally, this buffer must contain enough memory to store bookmarkdata.
The descendent must be able to construct a bookmark from this buffer.}
Function TSqlitePassDataset.AllocRecordBuffer: PChar;
Begin
{ test added to avoid memory leak
  when destroying component a [uncessary?] call is sent from TDataSet.SetBufListSize...}
if csDestroying in ComponentState then exit;
GetMem(Result, GetRecordSize);
End;


{procedure FreeRecordBuffer(var Buffer: PChar); virtual; abstract;
-----------------------------------------------------------------
Must free the memory allocated in the AllocRecordBuffer call.}
Procedure TSqlitePassDataset.FreeRecordBuffer(var Buffer: PChar);
Begin
FreeMem(Buffer, GetRecordSize);
End;


{procedure InternalInitRecord(Buffer: PChar); virtual; abstract;
---------------------------------------------------------------
This method is called to initialize a new field buffer when the dataset
is put into insert or append mode. We ask the InternalRecorset to add a new record }
Procedure TSqlitePassDataset.InternalInitRecord(Buffer: PChar);
begin
PInteger(Buffer)^ := Integer(FRecordset.CreateNewRecord(Fields));
{$IFDEF DEBUG}
FDatabase.Debugger.Display('InternalInitRecord : ' + IntToStr(TSqlitePassMemRecord(PInteger(Buffer)^).RecordIndex));
{$ENDIF}
End;


Function TSqlitePassDataset.GetCanModify: Boolean;
Begin
 Result := Active and (DatasetType in [dtTable, dtQuery, dtSqlDirect]) and (not ReadOnly);
End;

{function GetRecordSize: Word; virtual; abstract;
------------------------------------------------
Since we just use pointers to keep track of TSqlitePassMemRecords, the dataset just needs
to allocate enough size to hold a pointer value (i.e. longint or integer}
Function TSqlitePassDataset.GetRecordSize: Word;
begin
  Result := SizeOf(Pointer);
End;


{function GetRecord(Buffer: PChar; GetMode: TGetMode; DoCheck: Boolean): TGetResult; virtual; abstract;
-----------------------------------------------------------------------------------
This method must do 3 things:
1) Get the record data for the next/current/previous record, depending
   on the GetMode value. It should return
    grOK    if all was OK.
    grBOF   if the previous record was requested, and we are at the start.
    grEOF   if the next record was requested, and we are at the end.
    grError if an error occurred.

2) If DoCheck is True, and the result is grError, then an exception must be
    raised.

3) It should initialize bookmark data for this record with flag 'bfCurrent'
   This data can be stored in the buffer, if space was allocated for it with
   AllocRecordBuffer.
}
Function TSqlitePassDataset.GetRecord(Buffer: PChar; GetMode: TGetMode; DoCheck: Boolean): TGetResult;
Begin

case GetMode of
  gmCurrent: FRecordset.MoveBy(0);
  gmNext:    FRecordset.Next;
  gmPrior:   FRecordset.Prior;
  end;

Result := FRecordset.MoveState;

Case Result of
    grOk :  begin
            PInteger(Buffer)^ := Integer(FRecordset.CurrentRecord);
            FRecordset.CurrentRecord.BookMarkFlag := bfCurrent;
            { we update the calculated fields values }
            GetCalcFields(Buffer);
            end;
    grError:If DoCheck then DatabaseError(FDatasetName + ' : ' + Msg1040);
    end;
{$IFDEF DEBUG}
(*
 With FDatabase.Debugger do
      begin
      Display('[ --------- GetRecord ------- ]');
      Display('Getmode (gmCurrent, gmNext, gmPrior) = ' + IntToStr(Ord(GetMode)));
      Display('CurrentRecordIndex = ' + IntToStr(FRecordset.CurrentRecordIndex));
      Display('Dataset.state (dsInactive, dsBrowse, dsEdit, dsInsert, dsSetKey, dsCalcFields, dsFilter, dsNewValue, dsOldValue, dsCurValue, dsBlockRead, dsInternalCalc) = ' + IntToStr(Ord(Self.State)));
      Display('Dataset.Movestate (grOK, grBOF, grEOF, grError)= ' + IntToStr(Ord(Result)));
      Display('[ ------   <Get Record>  ---- ]');
      Display('');
      end;
 *)
{$ENDIF}
end;

Procedure TSqlitePassDataset.SetRecNo(Value: Integer);
begin
  CheckBrowseMode;
  FRecordset.CurrentRecordIndex := Pred(Value);
  DoBeforeScroll;
  Resync([rmCenter]);
  DoAfterScroll;
end;

{ ----- Filtering ----- }

{ Filtering with Master/Detail }

function TSqlitePassDataset.GetMasterDataSource: TDataSource;
begin
  Result := FMasterLink.DataSource;
end;

procedure TSqlitePassDataset.SetMasterDataSource(Value: TDataSource);
begin
  if IsLinkedTo(Value)
     then DatabaseError(Msg2026, Self);
  FMasterLink.DataSource := Value;
  if Value = nil
     then MasterFields := '';
end;

function TSqlitePassDataset.GetMasterFields: string;
begin
  Result := FMasterFields;
end;

procedure TSqlitePassDataset.SetMasterFields(const Value: string);
begin
FMasterFields := Value;

{ We parse MasterFields/DetailFields only when dataset.fields are defined }
if not (csLoading in ComponentState) and Active
   then ProcessMasterDetailFields;
end;

procedure TSqlitePassDataset.ProcessMasterDetailFields;
var
JoinedFieldEnd, PosEqual: Integer;
ParsedValue, DetailFieldName, CurrentJoinedFieldsNames, MasterFields: String;
begin
FDetailFields.Clear;
ParsedValue := '';
if FMasterFields <> '' then
   begin
   MasterFields := FMasterFields;
     Repeat
     JoinedFieldEnd := Pos(';', MasterFields);

     If JoinedFieldEnd = 0
        then JoinedFieldEnd := Length(MasterFields)+1;

     If JoinedFieldEnd > 0 then
        begin
        CurrentJoinedFieldsNames := System.Copy(MasterFields, 1, JoinedFieldEnd-1);
        System.Delete(MasterFields, 1, JoinedFieldEnd);
        PosEqual := Pos('=', CurrentJoinedFieldsNames);
        If PosEqual = 0
           then PosEqual := Length(CurrentJoinedFieldsNames)+1;
        If PosEqual > 0 then
           begin
           DetailFieldName := System.Copy(CurrentJoinedFieldsNames, PosEqual+1, JoinedFieldEnd - PosEqual);
           FDetailFields.Add(Fields.FindField(DetailFieldName));
           System.Delete(CurrentJoinedFieldsNames, PosEqual, JoinedFieldEnd - PosEqual);
           ParsedValue := ParsedValue + CurrentJoinedFieldsNames + ';';
           end;
        end;
      Until MasterFields = '';
    if ParsedValue[Length(ParsedValue)] = ';'
       then System.Delete(ParsedValue, Length(ParsedValue), 1);
    end;

FMasterLink.FieldNames := ParsedValue;
FSQLChanged := True;
RefreshActive;
end;

procedure TSqlitePassDataset.MasterChanged(Sender: TObject);
begin
 FSQLChanged := True;
 Refresh;
end;

procedure TSqlitePassDataset.MasterDisabled(Sender: TObject);
begin
 FSQLChanged := True;
 RefreshActive;
end;

procedure TSqlitePassDataset.DoOnNewRecord;
var
  i: Integer;
begin
{ Set default detail fields values from master fields if needed }
if MasterFields <> '' then
   begin
   For i := 0 to Pred(FDetailFields.Count)
       do TField(FDetailFields[i]).Value := TField(FMasterLink.Fields[i]).Value;
   end;

inherited DoOnNewRecord;
end;


{ Filtering with other filters }

procedure TSqlitePassDataset.CheckFiltered(ApplyFilter: Boolean = False);
begin
  FSQLChanged := True;
  ApplyFilter := ApplyFilter or (Filtered and ((Filter <> '') or (FFilterRecordLowerLimit <> 0) or (FFilterRecordUpperLimit <> 0)));
  If ApplyFilter
     then RefreshActive;
end;

{ This procedure is kept to give a public access to force applying a filter }
procedure TSqlitePassDataset.ApplyFilter;
begin
 Filtered := True;
 CheckFiltered(Filtered);
end;

procedure TSqlitePassDataset.SetFiltered(Value: Boolean);
begin
  If Value <> Filtered then
     begin
     Inherited SetFiltered(Value);
     CheckFiltered(True);
     end;
end;

procedure TSqlitePassDataset.SetFilterText(const Value: string);
begin
  Inherited SetFilterText(Value);
  CheckFiltered(Filtered);
end;

procedure TSqlitePassDataset.SetFFilterRecordLowerLimit(
  const Value: Cardinal);
begin
  FFilterRecordLowerLimit := Value;
  CheckFiltered(Filtered);
end;

procedure TSqlitePassDataset.SetFFilterRecordUpperLimit(
  const Value: Cardinal);
begin
  FFilterRecordUpperLimit := Value;
  CheckFiltered(Filtered);
end;

{ Implementation of virtual method TDataset.FindRecord }
function TSqlitePassDataset.FindRecord(Restart, GoForward: Boolean): Boolean;
begin
if GoForward
   then if Restart
           then First
           else Next
   else if Restart
           then Last
           else Prior;
Result := Found;
end;

{ --------- Locate / LookUp --------- }

function TSqlitePassDataset.Locate(const KeyFields: string;
  const KeyValues: Variant; Options: TLocateOptions): Boolean;
begin
  DoBeforeScroll;
  Result := LocateRecord(KeyFields, KeyValues, Options, True);
  if Result then
  begin
    Resync([rmExact, rmCenter]);
    DoAfterScroll;
  end;
end;


function TSqlitePassDataset.LocateRecord(const KeyFields: string;
  const KeyValues: Variant; Options: TLocateOptions;
  SyncCursor: Boolean): Boolean;
var
  StartIndex, RecordIndex, i, j: Integer;
  WhereStmt, LocateStmt: String;
  LocateFields: TList;
  CaseInsensitive: Boolean;

  { If the first PrimayKey is found checks the others primarykeys  }
  function CheckOthersPrimaryKeys(CurrentRecord, LocateCurrentRecord: Integer): Boolean;
  var
  i: Integer;
  begin
  Result := True;
  for i := 1 to Pred(FSQLSelectStmt.TableDefs.Count) do
     begin
     Result := FRecordset[CurrentRecord].FieldsValues[i] = FLocateRecordset[LocateCurrentRecord].FieldsValues[i];
     if Not Result then Break;
     end;
  end;

  { Locates the first matching record between Recordset and LocateRecordset }
  function FindPrimaryKey(StartIndex: Integer; PrimaryKeyValue: String; LocateCurrentRecord: Integer): Integer;
  var
  i: Integer;
  FirstPrimaryKeyFound, OthersPrimaryKeysFound: Boolean;
  begin
  Result := -1;
  for i := StartIndex to Pred(FRecordset.Count) do
     begin
     if (FRecordset[i].FieldsValues[0] = PrimaryKeyValue) and CheckOthersPrimaryKeys(i, LocateCurrentRecord) then
         begin
         Result := i;
         Break;
         end;
     end;
  end;

begin
  CheckBrowseMode;
  CursorPosChanged;
  FLocateTable.Clear;

  LocateFields := TList.Create;
  GetFieldList(LocateFields, KeyFields);

  { Create the additional "Where" clause}
  for i := 0 to Pred(LocateFields.Count) do
      LocateStmt := LocateStmt + TField(LocateFields[i]).FieldName + '= "' + KeyValues[I] + '" AND ';

  System.Delete(LocateStmt, Length(LocateStmt)-3, 4);

  FLocateSQL.FSQL.Assign(FSQLSelectStmt.FSQL);
  FLocateSQL.SplitSections;
  WhereStmt :=  FLocateSQL.FindSection(kwWhere);
  if WhereStmt = ''
     then WhereStmt := ' WHERE ' + LocateStmt
     else WhereStmt := ' AND ' + LocateStmt;

  FLocateSQL.ReplaceSection(kwWhere, WhereStmt);

  { Retrieves the corresponding records from Sqlite db }
  FLocateRecordset.Open(Database.Engine.Connection, FLocateSQL.FSQL.Text);

  { Matches records between Recordset and LocateRecordset }
  StartIndex := 0;
  for i := 0 to Pred(FLocateRecordset.Count) do
      begin
      RecordIndex := FindPrimaryKey(StartIndex, FLocateRecordset[i].FieldsValues[0], i);
      if RecordIndex > -1
         then FLocateTable.Add(Pointer(RecordIndex));
      If RecordIndex > StartIndex
         then StartIndex := RecordIndex + 1;
      end; // for i

   { Moves to the first matching record }   
   if FLocateTable.Count > 0
      then MoveBy(Integer(FLocateTable.Items[0]) - TSqlitePassMemRecord(PInteger(ActiveBuffer)^).RecordIndex);

  LocateFields.Free;
end;

{ Sorting }

procedure TSqlitePassDataset.SetFSorted(const Value: Boolean);
begin
  if FSortedBy = ''
     then FSorted := False
     else FSorted := Value;
  if FSorted
     then Sort;
end;

procedure TSqlitePassDataset.SetFSortedBy(const Value: String);
begin
  FSortedBy := Value;
  if FSortedBy = ''
     then FSorted := False;
  FSQLChanged := True;
  if Sorted
     then Sort;
end;

Procedure TSqlitePassDataset.Sort;
Begin
 RefreshActive;
End;

{ Returns a Unique field name, adding a number if necessary to duplicate names }
function TSqlitePassDataset.GetUniqueFieldName(FieldName: String): String;
var
i: Integer;
NewName: String;
begin
i := 0;
Result := FieldName;
if FieldDefs.IndexOf(FieldName) <> -1 // TODO use TFields.CheckFieldName instead ?
   then begin
        Repeat
        Inc(i);
        NewName := FieldName + '_' + IntToStr(i);
        Until FieldDefs.IndexOf(NewName) = -1;
        Result := NewName;
        end;
end;


{ ----- Queries params ----- Begin }

Procedure TSqlitePassDataset.SetParamsList(Value: TParams);
begin
 FParams.AssignValues(Value);
end;

Procedure TSqlitePassDataset.UpdateParamsList;
var
 NewParamsList: TParams;
begin
if not (csReading in ComponentState) then
    if ParamCheck or (csDesigning in ComponentState) then
    begin
        NewParamsList := TParams.Create(Self);
        try
            NewParamsList.ParseSQL(FSQLSelectStmt.SQL.Text, True);
            NewParamsList.AssignValues(FParams);
            FParams.Clear;
            FParams.Assign(NewParamsList);
        finally
            NewParamsList.Free;
        end;
    end;
end;

Function TSqlitePassDataset.GetParamsCount: Integer;
begin
 Result := FParams.Count;
end;

Procedure TSqlitePassDataset.DefineProperties(Filer: TFiler);
  Function WriteData: Boolean;
  begin
      if Filer.Ancestor <> nil then
          Result := not FParams.IsEqual(TSqlitePassDataset(Filer.Ancestor).FParams)
      else
          Result := FParams.Count > 0;
  end;
begin
  inherited DefineProperties(Filer);
  Filer.DefineProperty('ParamData', ReadParamData, WriteParamData, WriteData);
end;

Procedure TSqlitePassDataset.ReadParamData(Reader: TReader);
begin
  Reader.ReadValue;
  Reader.ReadCollection(FParams);
end;

Procedure TSqlitePassDataset.WriteParamData(Writer: TWriter);
begin
  Writer.WriteCollection(Params);
end;

{ ----- Queries Params ----- End }

procedure TSqlitePassDataset.FreeRecordPointers(Buffer: PChar);
begin
{ TODO : Doesn't seem necessary to implement this procedure
  since no call was found to this procedure in VCL  }
end;

procedure TSqlitePassDataset.AllocateBLOBPointers(Buffer: PChar);
begin
{ TODO : Doesn't seem necessary to implement this procedure
  since no call was found to this procedure in VCL  }
end;

procedure TSqlitePassDataset.FreeBlobPointers(Buffer: PChar);
begin
{ TODO : TSqlitePassDataset.FreeBlobPointers
  Doesn't seem necessary to implement this procedure
  since Blobs are cleared by TSqlitePassMemRecord.Clear
  and no call was found to this procedure in VCL  }
end;

Procedure TSqlitePassDataset.ClearCalcFields(Buffer: PChar);
begin
FRecordset.ClearCalcFields(TSqlitePassMemRecord(PInteger(Buffer)^));
{$IFDEF DEBUG}
FDatabase.Debugger.Display('[ --------- ClearCalcFields ------- ]');
{$ENDIF}

end;

function TSqlitePassDataset.GetRecordCount: Longint;
begin
   CheckActive;
   Result := FRecordset.Count;
end;

function TSqlitePassDataset.GetRecNo: Longint;
begin
 Result := FRecordset.CurrentRecordIndex;
end;

Function TSqlitePassDataset.CreateBlobStream(Field: TField; Mode: TBlobStreamMode): TStream;
Begin
 Result:=TSqlitePassBlobStream.Create(TBlobField(Field),Mode);
End;

procedure TSqlitePassDataset.CloseBlob(Field: TField);
begin
{ TODO : TSqlitePassDataset.CloseBlob
  Doesn't seem necessary to implement this procedure
  since Blobs are kept in memory by TSqliteRecordsetCache }
end;

procedure TSqlitePassDataset.SetFFieldDefsInfo(const Value: String);
begin
// Read only - Used to display infos in object inspector
end;


{ TSqlitePassDatasetIndex }

constructor TSqlitePassDatasetIndex.Create(Owner: TSqlitePassDatasetIndexDefs);
begin
inherited Create;
FIndexDefs := Owner;
end;

destructor TSqlitePassDatasetIndex.Destroy;
begin
FIndexDefs := nil;
inherited Destroy;
end;

procedure TSqlitePassDatasetIndex.DeleteIndex;
begin
 FindexDefs.FDataset.Database.Engine.Exec('Drop index ' + FIndexName);
 FIndexDefs.FDataset.FDatabase.IndexDefs.Refresh;
end;

procedure TSqlitePassDatasetIndex.Reindex;
begin
 FIndexDefs.FDataset.FDatabase.Engine.Exec('Reindex ' + FIndexName);
end;


{ TSqlitePassDatasetIndexDefs }

constructor TSqlitePassDatasetIndexDefs.Create(Owner: TSqlitePassDataset);
begin
 inherited Create;
 FDataset := Owner;
end;

destructor TSqlitePassDatasetIndexDefs.Destroy;
begin
 FDataset := nil;
 inherited Destroy;
end;

function TSqlitePassDatasetIndexDefs.GetItem(Index: Integer): TSqlitePassIndex;
begin
 Result:=TSqlitePassIndex(Inherited Items[Index]);
end;

procedure TSqlitePassDatasetIndexDefs.SetItem(Index: Integer;
  const Value: TSqlitePassIndex);
begin
 Put(Index,Value);
end;

procedure TSqlitePassDatasetIndexDefs.RefreshFromCache;
var
i: Integer;
TableName: String;
begin
{ Clears only the list, not items which are pointing to Database indexdefs }
Clear;
TableName := FDataset.FDatasetName;
if TableName <> ''
   then begin
        for i := 0 to Pred(FDataset.FDatabase.IndexDefs.count)
         do begin
            if AnsiCompareText(FDataset.FDatabase.IndexDefs[i].TableName, TableName) = 0
               then Add(FDataset.FDatabase.IndexDefs[i]);
            end;
   end;
end;


{ TBlobStream }

Constructor TSqlitePassBlobStream.Create(Field: TBlobField; Mode: TBlobStreamMode);
var
FActiveRec  : TSqlitePassMemRecord;
Begin
 FField := Field;
 FDataset := TSqlitePassDataset(FField.DataSet);
 FActiveRec := FDataset.GetActiveRecord;
 if FActiveRec = Nil then Exit;

 if FActiveRec.FieldsValues[Pred(Field.FieldNo)] = ''
    then begin
         FRecBlobStream := TMemoryStream.Create;
         FActiveRec.FieldsValues[Pred(Field.FieldNo)] := IntToStr(LongInt(FRecBlobStream));
         end
    else FRecBlobStream := TMemoryStream(StrToInt(FActiveRec.FieldsValues[Pred(Field.FieldNo)]));

 FOpened := True;

 Case Mode of
      bmRead  : LoadFromStream(FRecBlobStream);
      bmWrite : begin
                FRecBlobStream.Clear;
                FModified := True;
                end;
    end;
end;

Destructor TSqlitePassBlobStream.Destroy;
Begin
 if FModified then
   try
     SaveToStream(FRecBlobStream);
     FField.Modified := True;
     FDataSet.DataEvent(deFieldChange, Longint(FField));
     FOpened := False;
     FModified := False;
   Except
     Application.HandleException(Self);
   End;
inherited Destroy;
end;

function TSqlitePassBlobStream.Read(var Buffer; Count: Longint): Longint;
begin
 Result := 0;
 if FOpened
    then Result := inherited Read(Buffer, Count);
end;

Function TSqlitePassBlobStream.Write(const Buffer; Count: Longint): Longint;
Begin
 Result := 0;
 if FOpened then
    begin
    Result := inherited Write(Buffer, Count);
    FModified := True;
    end;
end;





